<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class DashboardFacilitiesFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getDadosLimpezaFinalizada($parametros, $listaOperadores, $filtroTipo)
    {

        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NRLOCALCLIENTE, FCM_CHECKLISTAGEND.NRCHECKLIST, FCM_CHECKLISTAGEND.CDOPERADOR, FCM_CHECKLISTAGEND.IDSITUACAOAGEND, CLIENTE.CDCLIENTE,
              TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTHORAAGEND, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTEPAI AS LOCALPAI, FCM_CHECKLIST.NMCHECKLIST, CLIENTE.NMFANTCLIE, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND,
              FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, OPERADOR.NMOPERADOR, FCM_EQUIPE.NMEQUIPE
                FROM 
                    FCM_CHECKLISTAGEND
                JOIN 
                    FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                JOIN 
                    FCM_CHECKLIST ON FCM_CHECKLISTAGEND.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                LEFT JOIN 
                    FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    OPERADOR ON FCM_CHECKLISTAGEND.CDOPERADOR = OPERADOR.CDOPERADOR
                LEFT JOIN 
                    FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    FCM_EQUIPE ON FCM_EQUIPE.NREQUIPE = FCM_EQUIPETAREFA.NREQUIPE
                    WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                        OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                        OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                            (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                            FROM FCM_EQUIPE
                            JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                            WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                        OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                       AND FCM_CHECKLISTAGEND.IDATIVO = 'S'
                       AND FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3'
                       AND FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1'
                       $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                  $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }
    
    public function getDadosTarefaAtrasada($parametros, $listaOperadores, $filtroTipo)
    {
        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NRLOCALCLIENTE, FCM_CHECKLISTAGEND.NRCHECKLIST, FCM_CHECKLISTAGEND.CDOPERADOR, FCM_CHECKLISTAGEND.IDSITUACAOAGEND, CLIENTE.CDCLIENTE,
              TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTHORAAGEND, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTEPAI AS LOCALPAI, FCM_CHECKLIST.NMCHECKLIST, CLIENTE.NMFANTCLIE, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND,
               CASE
                WHEN SYSDATE < FCM_CHECKLISTAGEND.DTHORAAGEND THEN ' '  
                WHEN TRUNC(SYSDATE) - TRUNC(FCM_CHECKLISTAGEND.DTHORAAGEND) < 1 THEN 
                    GREATEST(FLOOR((SYSDATE - FCM_CHECKLISTAGEND.DTHORAAGEND) * 24), 0) || ' horas ' || 
                    GREATEST(FLOOR((SYSDATE - FCM_CHECKLISTAGEND.DTHORAAGEND) * 24 * 60) - (FLOOR((SYSDATE - FCM_CHECKLISTAGEND.DTHORAAGEND) * 24) * 60), 0) || ' minutos'
                ELSE 
                    TRUNC(SYSDATE) - TRUNC(FCM_CHECKLISTAGEND.DTHORAAGEND) || ' dias'
                END AS TEMPO, 
                FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, OPERADOR.NMOPERADOR, FCM_EQUIPE.NMEQUIPE
                FROM 
                    FCM_CHECKLISTAGEND
                JOIN 
                    FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                JOIN 
                    FCM_CHECKLIST ON FCM_CHECKLISTAGEND.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                LEFT JOIN 
                    FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    OPERADOR ON FCM_CHECKLISTAGEND.CDOPERADOR = OPERADOR.CDOPERADOR
                LEFT JOIN 
                    FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    FCM_EQUIPE ON FCM_EQUIPE.NREQUIPE = FCM_EQUIPETAREFA.NREQUIPE
                    WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                        OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                        OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                            (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                            FROM FCM_EQUIPE
                            JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                            WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                        OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                       AND FCM_CHECKLISTAGEND.IDATIVO = 'S'
                       AND FCM_CHECKLISTAGEND.IDSITUACAOAGEND IN ('5', '1')
                       AND FCM_CHECKLISTAGEND.DTHORAAGEND <= SYSDATE
                       $filtroTipo";
    
        // Se houver filtro
        if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
            $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
        }
    
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getDadosTarefaConcluida($parametros, $listaOperadores, $filtroTipo)
    {

        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NRLOCALCLIENTE, FCM_CHECKLISTAGEND.NRCHECKLIST, FCM_CHECKLISTAGEND.CDOPERADOR, FCM_CHECKLISTAGEND.IDSITUACAOAGEND, CLIENTE.CDCLIENTE,
              TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTHORAAGEND, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTEPAI AS LOCALPAI, FCM_CHECKLIST.NMCHECKLIST, CLIENTE.NMFANTCLIE, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND,
              FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, OPERADOR.NMOPERADOR, FCM_EQUIPE.NMEQUIPE
                FROM 
                    FCM_CHECKLISTAGEND
                JOIN 
                    FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                JOIN 
                    FCM_CHECKLIST ON FCM_CHECKLISTAGEND.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                LEFT JOIN 
                    FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    OPERADOR ON FCM_CHECKLISTAGEND.CDOPERADOR = OPERADOR.CDOPERADOR
                LEFT JOIN 
                    FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    FCM_EQUIPE ON FCM_EQUIPE.NREQUIPE = FCM_EQUIPETAREFA.NREQUIPE
                    WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                        OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                        OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                            (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                            FROM FCM_EQUIPE
                            JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                            WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                        OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                       AND FCM_CHECKLISTAGEND.IDATIVO = 'S'
                       AND FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3' 
                       AND COALESCE(FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, '1') = '3'
                       $filtroTipo";

              // Se houver filtro
              if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
              }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getDadosTarefaEmAndamento($parametros, $listaOperadores, $filtroTipo)
    {

        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NRLOCALCLIENTE, FCM_CHECKLISTAGEND.NRCHECKLIST, FCM_CHECKLISTAGEND.CDOPERADOR, FCM_CHECKLISTAGEND.IDSITUACAOAGEND, CLIENTE.CDCLIENTE,
              TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTHORAAGEND, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTEPAI AS LOCALPAI, FCM_CHECKLIST.NMCHECKLIST, CLIENTE.NMFANTCLIE, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND,
              FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, OPERADOR.NMOPERADOR, FCM_EQUIPE.NMEQUIPE
                FROM 
                    FCM_CHECKLISTAGEND
                JOIN 
                    FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                JOIN 
                    FCM_CHECKLIST ON FCM_CHECKLISTAGEND.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                LEFT JOIN 
                    FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    OPERADOR ON FCM_CHECKLISTAGEND.CDOPERADOR = OPERADOR.CDOPERADOR
                LEFT JOIN 
                    FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    FCM_EQUIPE ON FCM_EQUIPE.NREQUIPE = FCM_EQUIPETAREFA.NREQUIPE
                    WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                        OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                        OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                            (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                            FROM FCM_EQUIPE
                            JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                            WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                        OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                       AND FCM_CHECKLISTAGEND.IDATIVO = 'S'
                       AND FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '2'
                       $filtroTipo";

              // Se houver filtro
              if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
              }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getDadosTarefaPendente($parametros, $listaOperadores, $filtroTipo)
    {

        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NRLOCALCLIENTE, FCM_CHECKLISTAGEND.NRCHECKLIST, FCM_CHECKLISTAGEND.CDOPERADOR, FCM_CHECKLISTAGEND.IDSITUACAOAGEND, CLIENTE.CDCLIENTE,
              TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTHORAAGEND, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTEPAI AS LOCALPAI, FCM_CHECKLIST.NMCHECKLIST, CLIENTE.NMFANTCLIE, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND,
              FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, OPERADOR.NMOPERADOR, FCM_EQUIPE.NMEQUIPE
                FROM 
                    FCM_CHECKLISTAGEND
                JOIN 
                    FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                JOIN 
                    FCM_CHECKLIST ON FCM_CHECKLISTAGEND.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                LEFT JOIN 
                    FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    OPERADOR ON FCM_CHECKLISTAGEND.CDOPERADOR = OPERADOR.CDOPERADOR
                LEFT JOIN 
                    FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    FCM_EQUIPE ON FCM_EQUIPE.NREQUIPE = FCM_EQUIPETAREFA.NREQUIPE
                    WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                        OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                        OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                            (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                            FROM FCM_EQUIPE
                            JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                            WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                        OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                       AND FCM_CHECKLISTAGEND.IDATIVO = 'S'
                       AND FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '1' 
                       AND FCM_CHECKLISTAGEND.DTHORAAGEND >= SYSDATE
                       $filtroTipo";

              // Se houver filtro
              if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
              }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getDadosTarefaCancelado($parametros, $listaOperadores, $filtroTipo)
    {

        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NRLOCALCLIENTE, FCM_CHECKLISTAGEND.NRCHECKLIST, FCM_CHECKLISTAGEND.CDOPERADOR, FCM_CHECKLISTAGEND.IDSITUACAOAGEND, CLIENTE.CDCLIENTE,
              TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTHORAAGEND, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTEPAI AS LOCALPAI, FCM_CHECKLIST.NMCHECKLIST, CLIENTE.NMFANTCLIE, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND,
              FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, OPERADOR.NMOPERADOR, FCM_EQUIPE.NMEQUIPE
                FROM 
                    FCM_CHECKLISTAGEND
                JOIN 
                    FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                JOIN 
                    FCM_CHECKLIST ON FCM_CHECKLISTAGEND.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                LEFT JOIN 
                    FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    OPERADOR ON FCM_CHECKLISTAGEND.CDOPERADOR = OPERADOR.CDOPERADOR
                LEFT JOIN 
                    FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    FCM_EQUIPE ON FCM_EQUIPE.NREQUIPE = FCM_EQUIPETAREFA.NREQUIPE
                    WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                        OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                        OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                            (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                            FROM FCM_EQUIPE
                            JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                            WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                        OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                       AND FCM_CHECKLISTAGEND.IDATIVO = 'S'
                       AND FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '4'
                       $filtroTipo";

              // Se houver filtro
              if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
              }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getDadosTarefaPendenteAprovacao($parametros, $listaOperadores, $filtroTipo)
    {

        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NRLOCALCLIENTE, FCM_CHECKLISTAGEND.NRCHECKLIST, FCM_CHECKLISTAGEND.CDOPERADOR, FCM_CHECKLISTAGEND.IDSITUACAOAGEND, CLIENTE.CDCLIENTE,
              TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTHORAAGEND, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTEPAI AS LOCALPAI, FCM_CHECKLIST.NMCHECKLIST, CLIENTE.NMFANTCLIE, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND,
              FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, OPERADOR.NMOPERADOR, FCM_EQUIPE.NMEQUIPE
                FROM 
                    FCM_CHECKLISTAGEND
                JOIN 
                    FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                JOIN 
                    FCM_CHECKLIST ON FCM_CHECKLISTAGEND.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                LEFT JOIN 
                    FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    OPERADOR ON FCM_CHECKLISTAGEND.CDOPERADOR = OPERADOR.CDOPERADOR
                LEFT JOIN 
                    FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    FCM_EQUIPE ON FCM_EQUIPE.NREQUIPE = FCM_EQUIPETAREFA.NREQUIPE
                    WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                        OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                        OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                            (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                            FROM FCM_EQUIPE
                            JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                            WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                        OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                       AND FCM_CHECKLISTAGEND.IDATIVO = 'S'
                       AND FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '6'
                       $filtroTipo";

              // Se houver filtro
              if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
              }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }


    public function getNaoConformidades($parametros, $listaOperadores) 
    {
        $sql = "SELECT CTGC.NRCRITERIOTAREFACHK, CTGC.NMCRITERIOTARGERCHK, COUNT(CTGC.NRCRITERIOTAREFACHK) AS QTDADE
              FROM FCM_CRITERIOTARGERCHK CTGC
              WHERE IDMARCADO = 'S'";

        // Se houver filtro por data
        if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
          $sql .= " AND CTGC.DTULTATU BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
        }
        
        $sql .= " GROUP BY CTGC.NRCRITERIOTAREFACHK, CTGC.NMCRITERIOTARGERCHK";
    
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getNaoConformidadesPorLocal($parametros, $listaOperadores) 
    {
        $sql = "SELECT TP.NMTIPOLOCALCLI, 
                   COUNT(CRIT.NRCRITERIOTAREFACHK) AS QTD
              FROM FCM_GERENCIAMENTOCHK GECHK
              JOIN FCM_TAREFAGERCHK TA
                ON GECHK.NRGERENCIAMENTOCHK = TA.NRGERENCIAMENTOCHK
              JOIN FCM_CRITERIOTARGERCHK CRIT
                ON CRIT.NRTAREFAGERCHK = TA.NRTAREFAGERCHK
              JOIN FCM_LOCALCLI LOC
                ON LOC.NRLOCALCLIENTE = GECHK.NRLOCALCLIENTE
              JOIN FCM_TIPOLOCALCLI TP
                ON TP.NRTIPOLOCALCLI = LOC.NRTIPOLOCALCLI
              WHERE CRIT.IDMARCADO = 'S'";

        // Se houver filtro por data
        if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
          $sql .= " AND CRIT.DTULTATU BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
        }
        
        $sql .= " GROUP BY TP.NMTIPOLOCALCLI";
    
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getTarefasPorCliente($parametros, $listaOperadores) 
    {
        $sql = "SELECT 
            CLIENTE.NMFANTCLIE, CLIENTE.CDCLIENTE,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3' AND COALESCE(FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, '1') = '3' THEN 1 END) AS total_concluidas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND IN ('5', '1') AND FCM_CHECKLISTAGEND.DTHORAAGEND <= SYSDATE THEN 1 END) AS total_atrasadas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3' AND COALESCE(FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, '1') = '1' THEN 1 END) AS total_finalizadas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '2' THEN 1 END) AS total_andamento,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '1' AND FCM_CHECKLISTAGEND.DTHORAAGEND >= SYSDATE THEN 1 END) AS total_pendentes,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '4' THEN 1 END) AS total_cancelados,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '6' THEN 1 END) AS total_pendentesaprovacao
        FROM 
            FCM_CHECKLISTAGEND
        JOIN 
            FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
        LEFT JOIN
            FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
        JOIN 
            CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
        LEFT JOIN 
            FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
        WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                    OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                    OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                        (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                        FROM FCM_EQUIPE
                        JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                        WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                    OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                    AND FCM_CHECKLISTAGEND.IDATIVO = 'S'";

        // Se houver filtro por data
        if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
          $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
        }
        
        $sql .= " GROUP BY 
        CLIENTE.NMFANTCLIE, CLIENTE.CDCLIENTE";
    
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getTarefasPorLocal($parametros, $listaOperadores) 
    {
        $sql = "SELECT 
            FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTE,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3' AND COALESCE(FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, '1') = '3' THEN 1 END) AS total_concluidas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND IN ('5', '1') AND FCM_CHECKLISTAGEND.DTHORAAGEND <= SYSDATE THEN 1 END) AS total_atrasadas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3' AND COALESCE(FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, '1') = '1' THEN 1 END) AS total_finalizadas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '2' THEN 1 END) AS total_andamento,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '1' AND FCM_CHECKLISTAGEND.DTHORAAGEND >= SYSDATE THEN 1 END) AS total_pendentes,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '4' THEN 1 END) AS total_cancelados,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '6' THEN 1 END) AS total_pendentesaprovacao
        FROM 
            FCM_CHECKLISTAGEND
        JOIN 
            FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
        LEFT JOIN
            FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
        LEFT JOIN 
            FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
        WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                    OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                    OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                        (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                        FROM FCM_EQUIPE
                        JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                        WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                    OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                    AND FCM_CHECKLISTAGEND.IDATIVO = 'S'";

        // Se houver filtro por data
        if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
          $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
        }
        
        $sql .= " GROUP BY 
        FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTE";
    
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }
    
    public function getTarefasPorChecklist($parametros, $listaOperadores) 
    {
        $sql = "SELECT 
            FCM_CHECKLIST.NMCHECKLIST, FCM_CHECKLIST.NRCHECKLIST,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3' AND COALESCE(FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, '1') = '3' THEN 1 END) AS total_concluidas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND IN ('5', '1') AND FCM_CHECKLISTAGEND.DTHORAAGEND <= SYSDATE THEN 1 END) AS total_atrasadas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3' AND COALESCE(FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, '1') = '1' THEN 1 END) AS total_finalizadas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '2' THEN 1 END) AS total_andamento,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '1' AND FCM_CHECKLISTAGEND.DTHORAAGEND >= SYSDATE THEN 1 END) AS total_pendentes,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '4' THEN 1 END) AS total_cancelados,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '6' THEN 1 END) AS total_pendentesaprovacao
        FROM 
            FCM_CHECKLISTAGEND
        JOIN
            FCM_CHECKLIST ON FCM_CHECKLISTAGEND.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
        LEFT JOIN
            FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
        LEFT JOIN 
            FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
        WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                    OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                    OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                        (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                        FROM FCM_EQUIPE
                        JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                        WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                    OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                    AND FCM_CHECKLISTAGEND.IDATIVO = 'S'";

        // Se houver filtro por data
        if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
          $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
        }
        
        $sql .= " GROUP BY 
        FCM_CHECKLIST.NMCHECKLIST, FCM_CHECKLIST.NRCHECKLIST";
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getVistoriasPendentes($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.NMGERENCIAMENTOCHK, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE,
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, FCM_GERENCIAMENTOCHK.DTHORAAGEND, FCM_CHECKLIST.NMCHECKLIST, OPERADOR.NMOPERADOR
                FROM 
                    FCM_GERENCIAMENTOCHK
                JOIN 
                    FCM_LOCALCLI ON FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN
                    OPERADOR ON FCM_GERENCIAMENTOCHK.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN
                    FCM_CHECKLIST ON FCM_GERENCIAMENTOCHK.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                WHERE 
                    (FCM_GERENCIAMENTOCHK.CDOPERADOR IN ({$hierarchyParam}) OR FCM_GERENCIAMENTOCHK.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' 
                AND 
                    FCM_GERENCIAMENTOCHK.DTHORAAGEND >= SYSDATE
                $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_GERENCIAMENTOCHK.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }
    
    public function getVistoriasAtrasadas($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.NMGERENCIAMENTOCHK, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE,
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, FCM_GERENCIAMENTOCHK.DTHORAAGEND, 
                    CASE
                        WHEN SYSDATE < FCM_GERENCIAMENTOCHK.DTHORAAGEND THEN ' '  
                        WHEN TRUNC(SYSDATE) - TRUNC(FCM_GERENCIAMENTOCHK.DTHORAAGEND) < 1 THEN 
                            GREATEST(FLOOR((SYSDATE - FCM_GERENCIAMENTOCHK.DTHORAAGEND) * 24), 0) || ' horas ' || 
                            GREATEST(FLOOR((SYSDATE - FCM_GERENCIAMENTOCHK.DTHORAAGEND) * 24 * 60) - (FLOOR((SYSDATE - FCM_GERENCIAMENTOCHK.DTHORAAGEND) * 24) * 60), 0) || ' minutos'
                        ELSE 
                            TRUNC(SYSDATE) - TRUNC(FCM_GERENCIAMENTOCHK.DTHORAAGEND) || ' dias'
                        END AS TEMPO, FCM_CHECKLIST.NMCHECKLIST, OPERADOR.NMOPERADOR
                FROM 
                    FCM_GERENCIAMENTOCHK
                JOIN 
                    FCM_LOCALCLI ON FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN
                    OPERADOR ON FCM_GERENCIAMENTOCHK.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN
                    FCM_CHECKLIST ON FCM_GERENCIAMENTOCHK.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                WHERE 
                    (FCM_GERENCIAMENTOCHK.CDOPERADOR IN ({$hierarchyParam}) OR FCM_GERENCIAMENTOCHK.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' 
                AND 
                    FCM_GERENCIAMENTOCHK.DTHORAAGEND <= SYSDATE
                $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_GERENCIAMENTOCHK.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getVistoriasAvaliadas($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.NMGERENCIAMENTOCHK, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE,
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, FCM_GERENCIAMENTOCHK.DTHORAAGEND, FCM_CHECKLIST.NMCHECKLIST, OPERADOR.NMOPERADOR
                FROM 
                    FCM_GERENCIAMENTOCHK
                JOIN 
                    FCM_LOCALCLI ON FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN
                    OPERADOR ON FCM_GERENCIAMENTOCHK.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN
                    FCM_CHECKLIST ON FCM_GERENCIAMENTOCHK.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                WHERE 
                    (FCM_GERENCIAMENTOCHK.CDOPERADOR IN ({$hierarchyParam}) OR FCM_GERENCIAMENTOCHK.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '3'
                $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_GERENCIAMENTOCHK.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getVistoriasEmAndamento($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.NMGERENCIAMENTOCHK, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE,
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, FCM_GERENCIAMENTOCHK.DTHORAAGEND, FCM_CHECKLIST.NMCHECKLIST, OPERADOR.NMOPERADOR
                FROM 
                    FCM_GERENCIAMENTOCHK
                JOIN 
                    FCM_LOCALCLI ON FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN
                    OPERADOR ON FCM_GERENCIAMENTOCHK.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN
                    FCM_CHECKLIST ON FCM_GERENCIAMENTOCHK.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                WHERE 
                    (FCM_GERENCIAMENTOCHK.CDOPERADOR IN ({$hierarchyParam}) OR FCM_GERENCIAMENTOCHK.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK IN ('4', '2')
                $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_GERENCIAMENTOCHK.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getVistoriasCancelado($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.NMGERENCIAMENTOCHK, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE,
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, FCM_GERENCIAMENTOCHK.DTHORAAGEND, FCM_CHECKLIST.NMCHECKLIST, OPERADOR.NMOPERADOR
                FROM 
                    FCM_GERENCIAMENTOCHK
                JOIN 
                    FCM_LOCALCLI ON FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN
                    OPERADOR ON FCM_GERENCIAMENTOCHK.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN
                    FCM_CHECKLIST ON FCM_GERENCIAMENTOCHK.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                WHERE 
                    (FCM_GERENCIAMENTOCHK.CDOPERADOR IN ({$hierarchyParam}) OR FCM_GERENCIAMENTOCHK.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '5'
                $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_GERENCIAMENTOCHK.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getVistoriasPorCliente($parametros, $hierarchyParam)
    {
        $sql = "SELECT 
                    CLIENTE.NMFANTCLIE, CLIENTE.CDCLIENTE,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' AND FCM_GERENCIAMENTOCHK.DTHORAAGEND >= SYSDATE THEN 1 END) AS total_pendentes,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' AND FCM_GERENCIAMENTOCHK.DTHORAAGEND <= SYSDATE THEN 1 END) AS total_atrasadas,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '3' THEN 1 END) AS total_avaliadas,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK IN ('4', '2') THEN 1 END) AS total_andamento,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '5' THEN 1 END) AS total_cancelado
                FROM 
                    FCM_GERENCIAMENTOCHK
                JOIN 
                    FCM_LOCALCLI ON FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                WHERE (
                    (FCM_GERENCIAMENTOCHK.CDOPERADOR IN ($hierarchyParam)) 
                OR (FCM_GERENCIAMENTOCHK.CDOPERINCLUSAO = :CDOPERADOR AND FCM_GERENCIAMENTOCHK.IDATIVO = 'S')
                    )";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_GERENCIAMENTOCHK.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }
                
                $sql .= " GROUP BY 
                CLIENTE.NMFANTCLIE, CLIENTE.CDCLIENTE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getVistoriasPorChecklist($parametros, $hierarchyParam)
    {
        $sql = "SELECT 
                    FCM_CHECKLIST.NMCHECKLIST, FCM_CHECKLIST.NRCHECKLIST,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' AND FCM_GERENCIAMENTOCHK.DTHORAAGEND >= SYSDATE THEN 1 END) AS total_pendentes,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' AND FCM_GERENCIAMENTOCHK.DTHORAAGEND <= SYSDATE THEN 1 END) AS total_atrasadas,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '3' THEN 1 END) AS total_avaliadas,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK IN ('4', '2') THEN 1 END) AS total_andamento,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '5' THEN 1 END) AS total_cancelado
                FROM 
                    FCM_GERENCIAMENTOCHK
                JOIN
                    FCM_CHECKLIST ON FCM_GERENCIAMENTOCHK.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                WHERE (
                    (FCM_GERENCIAMENTOCHK.CDOPERADOR IN ($hierarchyParam)) 
                OR (FCM_GERENCIAMENTOCHK.CDOPERINCLUSAO = :CDOPERADOR AND FCM_GERENCIAMENTOCHK.IDATIVO = 'S')
                    )";

                    
                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_GERENCIAMENTOCHK.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }
                
                $sql .= " GROUP BY 
                FCM_CHECKLIST.NMCHECKLIST, FCM_CHECKLIST.NRCHECKLIST";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getVistoriasPorLocal($parametros, $hierarchyParam)
    {
        $sql = "SELECT 
                    FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTE,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' AND FCM_GERENCIAMENTOCHK.DTHORAAGEND >= SYSDATE THEN 1 END) AS total_pendentes,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' AND FCM_GERENCIAMENTOCHK.DTHORAAGEND <= SYSDATE THEN 1 END) AS total_atrasadas,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '3' THEN 1 END) AS total_avaliadas,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK IN ('4', '2') THEN 1 END) AS total_andamento,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '5' THEN 1 END) AS total_cancelado
                FROM 
                    FCM_GERENCIAMENTOCHK
                JOIN 
                    FCM_LOCALCLI ON FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                WHERE (
                    (FCM_GERENCIAMENTOCHK.CDOPERADOR IN ($hierarchyParam)) 
                OR (FCM_GERENCIAMENTOCHK.CDOPERINCLUSAO = :CDOPERADOR AND FCM_GERENCIAMENTOCHK.IDATIVO = 'S')
                    )";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_GERENCIAMENTOCHK.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }
                
                $sql .= " GROUP BY 
                FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getAuditoriasAtrasadas($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_AUDITORIA.NRAUDITORIA, FCM_AUDITORIA.NMAUDITORIA, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE, FCM_MODELOAUDIT.NMMODELOAUDIT,
                    FCM_AUDITORIA.IDSITUACAOAUDIT, FCM_AUDITORIA.DTHORAAUDIT, FCM_AUDITORIA.PERCENTUALCONCLUSAO, FCM_AUDITORIA.NOTATOTAL, OPERADOR.NMOPERADOR
                FROM 
                    FCM_AUDITORIA
                JOIN 
                    FCM_LOCALCLI ON FCM_AUDITORIA.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN
                    OPERADOR ON FCM_AUDITORIA.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    FCM_MODELOAUDIT ON FCM_AUDITORIA.NRMODELOAUDIT = FCM_MODELOAUDIT.NRMODELOAUDIT
                WHERE 
                    (FCM_AUDITORIA.CDOPERADOR IN ({$hierarchyParam}) OR FCM_AUDITORIA.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_AUDITORIA.IDSITUACAOAUDIT = '1' 
                AND 
                    FCM_AUDITORIA.DTHORAAUDIT <= SYSDATE
                    $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_AUDITORIA.DTHORAAUDIT BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getAuditoriasEmAndamento($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_AUDITORIA.NRAUDITORIA, FCM_AUDITORIA.NMAUDITORIA, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE, FCM_MODELOAUDIT.NMMODELOAUDIT,
                    FCM_AUDITORIA.IDSITUACAOAUDIT, FCM_AUDITORIA.DTHORAAUDIT, FCM_AUDITORIA.PERCENTUALCONCLUSAO, FCM_AUDITORIA.NOTATOTAL, OPERADOR.NMOPERADOR
                FROM 
                    FCM_AUDITORIA
                JOIN 
                    FCM_LOCALCLI ON FCM_AUDITORIA.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN
                    OPERADOR ON FCM_AUDITORIA.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    FCM_MODELOAUDIT ON FCM_AUDITORIA.NRMODELOAUDIT = FCM_MODELOAUDIT.NRMODELOAUDIT
                WHERE 
                    (FCM_AUDITORIA.CDOPERADOR IN ({$hierarchyParam}) OR FCM_AUDITORIA.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_AUDITORIA.IDSITUACAOAUDIT = '2'
                    $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_AUDITORIA.DTHORAAUDIT BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getAuditoriasConcluidas($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_AUDITORIA.NRAUDITORIA, FCM_AUDITORIA.NMAUDITORIA, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE, FCM_MODELOAUDIT.NMMODELOAUDIT,
                    FCM_AUDITORIA.IDSITUACAOAUDIT, FCM_AUDITORIA.DTHORAAUDIT, FCM_AUDITORIA.PERCENTUALCONCLUSAO, FCM_AUDITORIA.NOTATOTAL, OPERADOR.NMOPERADOR
                FROM 
                    FCM_AUDITORIA
                JOIN 
                    FCM_LOCALCLI ON FCM_AUDITORIA.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN
                    OPERADOR ON FCM_AUDITORIA.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN 
                    FCM_MODELOAUDIT ON FCM_AUDITORIA.NRMODELOAUDIT = FCM_MODELOAUDIT.NRMODELOAUDIT
                WHERE 
                    (FCM_AUDITORIA.CDOPERADOR IN ({$hierarchyParam}) OR FCM_AUDITORIA.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_AUDITORIA.IDSITUACAOAUDIT = '3'
                    $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_AUDITORIA.DTHORAAUDIT BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getAuditoriasPendentes($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_AUDITORIA.NRAUDITORIA, FCM_AUDITORIA.NMAUDITORIA, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE, FCM_MODELOAUDIT.NMMODELOAUDIT,
                    FCM_AUDITORIA.IDSITUACAOAUDIT, FCM_AUDITORIA.DTHORAAUDIT, FCM_AUDITORIA.PERCENTUALCONCLUSAO, FCM_AUDITORIA.NOTATOTAL, OPERADOR.NMOPERADOR
                FROM 
                    FCM_AUDITORIA
                JOIN 
                    FCM_LOCALCLI ON FCM_AUDITORIA.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN
                    OPERADOR ON FCM_AUDITORIA.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN 
                    FCM_MODELOAUDIT ON FCM_AUDITORIA.NRMODELOAUDIT = FCM_MODELOAUDIT.NRMODELOAUDIT
                WHERE 
                    (FCM_AUDITORIA.CDOPERADOR IN ({$hierarchyParam}) OR FCM_AUDITORIA.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_AUDITORIA.IDSITUACAOAUDIT = '1'
                AND 
                    FCM_AUDITORIA.DTHORAAUDIT >= SYSDATE
                    $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_AUDITORIA.DTHORAAUDIT BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getAuditoriasPorLocal($parametros, $hierarchyParam)
    {
        $sql = "SELECT 
                    FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTE,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '2' THEN 1 END) AS total_andamento,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '1' AND FCM_AUDITORIA.DTHORAAUDIT <= SYSDATE THEN 1 END) AS total_atrasadas,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '3' THEN 1 END) AS total_concluidas,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '1' AND FCM_AUDITORIA.DTHORAAUDIT >= SYSDATE THEN 1 END) AS total_pendentes
                FROM 
                    FCM_AUDITORIA
                JOIN 
                    FCM_LOCALCLI ON FCM_AUDITORIA.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                WHERE (
                    (FCM_AUDITORIA.CDOPERADOR IN ($hierarchyParam)) 
                OR (FCM_AUDITORIA.CDOPERINCLUSAO = :CDOPERADOR AND FCM_AUDITORIA.IDATIVO = 'S')
                    )";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_AUDITORIA.DTHORAAUDIT BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }
                
                $sql .= " GROUP BY 
                FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getAuditoriasPorCliente($parametros, $hierarchyParam)
    {
        $sql = "SELECT 
                    CLIENTE.NMFANTCLIE, CLIENTE.CDCLIENTE, 
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '2' THEN 1 END) AS total_andamento,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '1' AND FCM_AUDITORIA.DTHORAAUDIT <= SYSDATE THEN 1 END) AS total_atrasadas,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '3' THEN 1 END) AS total_concluidas,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '1' AND FCM_AUDITORIA.DTHORAAUDIT >= SYSDATE THEN 1 END) AS total_pendentes
                FROM 
                    FCM_AUDITORIA
                JOIN 
                    FCM_LOCALCLI ON FCM_AUDITORIA.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                WHERE (
                    (FCM_AUDITORIA.CDOPERADOR IN ($hierarchyParam)) 
                OR (FCM_AUDITORIA.CDOPERINCLUSAO = :CDOPERADOR AND FCM_AUDITORIA.IDATIVO = 'S')
                    )";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_AUDITORIA.DTHORAAUDIT BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }
                
                $sql .= " GROUP BY 
                CLIENTE.NMFANTCLIE, CLIENTE.CDCLIENTE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getAuditoriasPorModelo($parametros, $hierarchyParam)
    {
        $sql = "SELECT 
                    FCM_MODELOAUDIT.NMMODELOAUDIT, FCM_MODELOAUDIT.NRMODELOAUDIT,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '2' THEN 1 END) AS total_andamento,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '1' AND FCM_AUDITORIA.DTHORAAUDIT <= SYSDATE THEN 1 END) AS total_atrasadas,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '3' THEN 1 END) AS total_concluidas,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '1' AND FCM_AUDITORIA.DTHORAAUDIT >= SYSDATE THEN 1 END) AS total_pendentes
                FROM 
                    FCM_AUDITORIA
                JOIN 
                    FCM_MODELOAUDIT ON FCM_AUDITORIA.NRMODELOAUDIT = FCM_MODELOAUDIT.NRMODELOAUDIT
                WHERE (
                    (FCM_AUDITORIA.CDOPERADOR IN ($hierarchyParam)) 
                OR (FCM_AUDITORIA.CDOPERINCLUSAO = :CDOPERADOR AND FCM_AUDITORIA.IDATIVO = 'S')
                    )";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_AUDITORIA.DTHORAAUDIT BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }
                
                $sql .= " GROUP BY 
                FCM_MODELOAUDIT.NMMODELOAUDIT, FCM_MODELOAUDIT.NRMODELOAUDIT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getOrdens($parametros)
    {
        $sql = "SELECT 
                    COUNT(CASE WHEN IDSITUACAO = '1' THEN 1 END) AS total_pendentes,
                    COUNT(CASE WHEN IDSITUACAO = '2' THEN 1 END) AS total_andamento,
                    COUNT(CASE WHEN IDSITUACAO = '3' THEN 1 END) AS total_finalizadas,
                    COUNT(CASE WHEN IDSITUACAO = '4' THEN 1 END) AS total_pausadas,
                    COUNT(CASE WHEN IDSITUACAO = '5' THEN 1 END) AS total_recusadas
                FROM 
                    FCM_ORDEMSERV
                ";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " WHERE FCM_ORDEMSERV.DTORDEMSERV BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getDadosOrdens($params, $filtroTipo)
    {
        $sql = "SELECT 
                  OSE.NRORDEMSERV,
                  TO_CHAR(OSE.DTORDEMSERV, 'DD/MM') AS DTORDEMSERV,
                  LPAD(EXTRACT(HOUR FROM OSE.DURACAOORDEM) + (EXTRACT(DAY FROM OSE.DURACAOORDEM) * 24), 2, '0') || ':' ||
                  LPAD(EXTRACT(MINUTE FROM OSE.DURACAOORDEM), 2, '0') || ':' ||
                  LPAD(EXTRACT(SECOND FROM OSE.DURACAOORDEM), 2, '0') AS DURACAO,
                  OSE.IDSITUACAO,
                  EP.NMEQUIPAMENTO,
                  CAT.NMCATEGORIAMANUT,
                  TM.NMTIPOMANUTENCAO,
                  P.NMPRIORIDADEMANUT,
                  P.DSCORPRIORIDADE,
                  OSE.NREQUIPAMENTO,
                  OSE.NRCATEGORIAMANUT,
                  OSE.NRTIPOMANUTENCAO,
                  OSE.NRPRIORIDADEMANUT,
                  OSE.DSORDEMSERV,
                  OSE.NMORDEMOSERV,
                  OSE.NRSOLICITACAOSERV,
                  SS.DTSOLICITACAOSERV,
                  OP.NMOPERADOR,
                  EQP.NREQUIPE,
                  EQP.NMEQUIPE,
                  CLI.NRLOCALCLIENTE,
                  CLI.NMLOCALCLIENTE,
                  CLIE.NMFANTCLIE,
                  
                  -- Lista de funcionários responsáveis pela Ordem de Serviço
                  NVL((
                      SELECT LISTAGG(OPERADOR.NMOPERADOR, ', ') 
                          WITHIN GROUP (ORDER BY FCM_RESPONSAVELORDEMS.CDOPERADOR)
                      FROM FCM_RESPONSAVELORDEMS
                      JOIN OPERADOR 
                          ON FCM_RESPONSAVELORDEMS.CDOPERADOR = OPERADOR.CDOPERADOR
                      WHERE OSE.NRORDEMSERV = FCM_RESPONSAVELORDEMS.NRORDEMSERV
                  ), null) AS OPERADORRESPONSAVEL,

                 -- Lista de funcionarios externos responsaveis pela ordem de serviço
                  NVL((
                      SELECT LISTAGG(FCM_PROFISSIONALEXTERNO.NMPROFISSIONALEXTERNO, ', ') 
                          WITHIN GROUP (ORDER BY FCM_RESPONSAVELORDEMS.NRPROFISSIONALEXTERNO)
                      FROM FCM_RESPONSAVELORDEMS
                      JOIN FCM_PROFISSIONALEXTERNO 
                          ON FCM_RESPONSAVELORDEMS.NRPROFISSIONALEXTERNO = FCM_PROFISSIONALEXTERNO.NRPROFISSIONALEXTERNO
                      WHERE OSE.NRORDEMSERV = FCM_RESPONSAVELORDEMS.NRORDEMSERV
                  ), null) AS PROFISSIONALEXTERNO,

                  -- Lista de escopos de manutenção relacionados à Ordem de Serviço
                  NVL((
                      SELECT LISTAGG(FCM_ESCOPOMANUT.NMESCOPOMANUT, ', ') 
                          WITHIN GROUP (ORDER BY FCM_ESCOPOMANUT.NRESCOPOMANUT)
                      FROM FCM_RELESCORDEMSERV
                      JOIN FCM_ESCOPOMANUT 
                          ON FCM_RELESCORDEMSERV.NRESCOPOMANUT = FCM_ESCOPOMANUT.NRESCOPOMANUT
                      WHERE OSE.NRORDEMSERV = FCM_RELESCORDEMSERV.NRORDEMSERV
                  ), null) AS NMESCOPOMANUT,

                  -- Descrição do status da Ordem de Serviço
                  CASE OSE.IDSITUACAO 
                      WHEN '1' THEN 'Pendente'
                      WHEN '2' THEN 'Em Andamento'
                      WHEN '3' THEN 'Finalizada'
                      WHEN '4' THEN 'Recusada'
                      WHEN '5' THEN 'Pausada'
                      ELSE NULL 
                  END AS NMIDSITUACAO

              FROM FCM_ORDEMSERV OSE

              -- Junções com outras tabelas
              LEFT JOIN FCM_SOLICITACAOSERV SS 
                  ON OSE.NRSOLICITACAOSERV = SS.NRSOLICITACAOSERV

              LEFT JOIN FCM_PRIORIDADEMANUT P 
                  ON P.NRPRIORIDADEMANUT = OSE.NRPRIORIDADEMANUT

              LEFT JOIN FCM_EQUIPE EQP 
                  ON EQP.NREQUIPE = OSE.NREQUIPE

              LEFT JOIN FCM_EQUIPAMENTO EP 
                  ON EP.NREQUIPAMENTO = OSE.NREQUIPAMENTO

              LEFT JOIN FCM_CATEGORIAMANUT CAT 
                  ON CAT.NRCATEGORIAMANUT = OSE.NRCATEGORIAMANUT

              LEFT JOIN FCM_TIPOMANUTENCAO TM 
                  ON TM.NRTIPOMANUTENCAO = OSE.NRTIPOMANUTENCAO

              LEFT JOIN OPERADOR OP 
                  ON SS.CDOPERSOLICITANTE = OP.CDOPERADOR

              LEFT JOIN FCM_LOCALCLI CLI
                  ON CLI.NRLOCALCLIENTE = OSE.NRLOCALCLIENTE

              LEFT JOIN CLIENTE CLIE
                  ON CLIE.CDCLIENTE = CLI.CDCLIENTE

              LEFT JOIN FCM_RESPONSAVELORDEMS RESPOP
                  ON OSE.NRORDEMSERV = RESPOP.NRORDEMSERV

              WHERE OSE.NRORDEMSERV IS NOT NULL

                " . $filtroTipo;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function getOrdensEquipamento($parametros)
    {
        $sql = "SELECT FCM_EQUIPAMENTO.NREQUIPAMENTO, FCM_EQUIPAMENTO.NMEQUIPAMENTO,
                    COUNT(CASE WHEN FCM_ORDEMSERV.IDSITUACAO = '1' THEN 1 END) AS total_pendentes,
                    COUNT(CASE WHEN FCM_ORDEMSERV.IDSITUACAO = '2' THEN 1 END) AS total_andamento,
                    COUNT(CASE WHEN FCM_ORDEMSERV.IDSITUACAO = '3' THEN 1 END) AS total_finalizadas,
                    COUNT(CASE WHEN FCM_ORDEMSERV.IDSITUACAO = '4' THEN 1 END) AS total_pausadas,
                    COUNT(CASE WHEN FCM_ORDEMSERV.IDSITUACAO = '5' THEN 1 END) AS total_recusadas
                FROM 
                    FCM_ORDEMSERV
                INNER JOIN FCM_EQUIPAMENTO 
                    ON FCM_EQUIPAMENTO.NREQUIPAMENTO = FCM_ORDEMSERV.NREQUIPAMENTO
                ";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_ORDEMSERV.DTORDEMSERV BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }
                $sql .= "GROUP BY FCM_EQUIPAMENTO.NREQUIPAMENTO, FCM_EQUIPAMENTO.NMEQUIPAMENTO";


        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getOrdensLocal($parametros)
    {
        $sql = "SELECT FCM_LOCALCLI.NRLOCALCLIENTE, FCM_LOCALCLI.NMLOCALCLIENTE,
                    COUNT(CASE WHEN IDSITUACAO = '1' THEN 1 END) AS total_pendentes,
                    COUNT(CASE WHEN IDSITUACAO = '2' THEN 1 END) AS total_andamento,
                    COUNT(CASE WHEN IDSITUACAO = '3' THEN 1 END) AS total_finalizadas,
                    COUNT(CASE WHEN IDSITUACAO = '4' THEN 1 END) AS total_pausadas,
                    COUNT(CASE WHEN IDSITUACAO = '5' THEN 1 END) AS total_recusadas
                FROM 
                    FCM_ORDEMSERV
                INNER JOIN FCM_LOCALCLI 
                    ON FCM_LOCALCLI.NRLOCALCLIENTE = FCM_ORDEMSERV.NRLOCALCLIENTE
                ";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " WHERE FCM_ORDEMSERV.DTORDEMSERV BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }
                $sql .= "GROUP BY FCM_LOCALCLI.NRLOCALCLIENTE, FCM_LOCALCLI.NMLOCALCLIENTE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getSolicitacoes($parametros)
    {
        $sql = "SELECT
                    COUNT(CASE WHEN IDSITUACAO = '1' THEN 1 END) AS total_abertas,
                    COUNT(CASE WHEN IDSITUACAO = '5' THEN 1 END) AS total_recusadas
                FROM 
                    FCM_SOLICITACAOSERV";

                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " WHERE FCM_SOLICITACAOSERV.DTSOLICITACAOSERV BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getDadosSolicitacoes($parametros, $filtroTipo)
    {
        $sql = "SELECT 
                    SOLSERV.NRSOLICITACAOSERV, 
                    TO_CHAR(SOLSERV.DTSOLICITACAOSERV, 'DD/MM/YY') AS DTSOLICITACAOSERV, 
                    CATMANUT.NMCATEGORIAMANUT, 
                    EQUIP.NMEQUIPAMENTO,
                    SOLSERV.DSSOLICITACAOSERV, 
                    NVL(OPER.NMOPERADOR, '') AS NMOPERADOR,
                    SOLSERV.NRFUNCSOLICITANTE, 
                    SOLSERV.IDSITUACAO,
                    SOLSERV.NREQUIPAMENTO, 
                    SOLSERV.NRCATEGORIAMANUT, 
                    FCM_GERENCIAMENTOCHK.NMGERENCIAMENTOCHK,    
                    FCM_CHECKLISTAGEND.NMCHECKLISTAGEND,
                    SOLSERV.NRLOCALCLIENTE,
                    FCM_LOCALCLI.NMLOCALCLIENTE,
                    FCM_LOCALCLI.CDCLIENTE,
                    CLIENTE.NMFANTCLIE,
                    TO_CHAR(
                        COALESCE(
                            FCM_GERENCIAMENTOCHK.DTHORAAGEND, 
                            FCM_CHECKLISTAGEND.DTHORAAGEND
                        ), 
                        'DD/MM/YY HH24:MI'
                    ) AS DTHORAAGEND,
                    SOLSERV.NRIMPACTOMANUT, 
                    IMPMANUT.NMIMPACTOMANUT, 
                    SOLSERV.NRPRIORIDADEMANUT, 
                    SOLSERV.NMSOLICITACAOSERV, 
                    SOLSERV.NRGERENCIAMENTOCHK, 
                    SOLSERV.NRCHECKLISTAGEND,
                    (
                        SELECT 
                            LISTAGG(FCM_IMGSOLICITACAOSERV.DSCAMINHO, ', ') 
                            WITHIN GROUP (ORDER BY FCM_IMGSOLICITACAOSERV.DSCAMINHO)
                        FROM 
                            FCM_IMGSOLICITACAOSERV
                        WHERE 
                            SOLSERV.NRSOLICITACAOSERV = FCM_IMGSOLICITACAOSERV.NRSOLICITACAOSERV
                    ) AS DSCAMINHOABC,
                    (
                        SELECT 
                            LISTAGG(ESCMANUT.NRESCOPOMANUT, ', ') 
                            WITHIN GROUP (ORDER BY ESCMANUT.NRESCOPOMANUT)
                        FROM 
                            FCM_RELESCSOLICITASERV RELESCOSOL
                        JOIN 
                            FCM_ESCOPOMANUT ESCMANUT
                            ON RELESCOSOL.NRESCOPOMANUT = ESCMANUT.NRESCOPOMANUT
                        WHERE 
                            SOLSERV.NRSOLICITACAOSERV = RELESCOSOL.NRSOLICITACAOSERV
                    ) AS NRESCOPOMANUT,
                    (
                        SELECT 
                            LISTAGG(ESCMANUT.NMESCOPOMANUT, ', ') 
                            WITHIN GROUP (ORDER BY ESCMANUT.NRESCOPOMANUT)
                        FROM 
                            FCM_RELESCSOLICITASERV RELESCOSOL
                        JOIN 
                            FCM_ESCOPOMANUT ESCMANUT
                            ON RELESCOSOL.NRESCOPOMANUT = ESCMANUT.NRESCOPOMANUT
                        WHERE 
                            SOLSERV.NRSOLICITACAOSERV = RELESCOSOL.NRSOLICITACAOSERV
                    ) AS NMESCOPOMANUT,
                    CASE SOLSERV.IDSITUACAO
                        WHEN '1' THEN 'Aberta'
                        WHEN '2' THEN 'Em Andamento'
                        WHEN '3' THEN 'Aguardando vistoria'
                        WHEN '4' THEN 'Concluída'
                        WHEN '5' THEN 'Recusada'
                        ELSE NULL
                    END AS SITUACAO
                FROM 
                    FCM_SOLICITACAOSERV SOLSERV
                LEFT JOIN 
                    FCM_EQUIPAMENTO EQUIP
                    ON SOLSERV.NREQUIPAMENTO = EQUIP.NREQUIPAMENTO
                LEFT JOIN 
                    FCM_CATEGORIAMANUT CATMANUT
                    ON SOLSERV.NRCATEGORIAMANUT = CATMANUT.NRCATEGORIAMANUT
                LEFT JOIN 
                    OPERADOR OPER
                    ON SOLSERV.CDOPERSOLICITANTE = OPER.CDOPERADOR
                LEFT JOIN 
                    FCM_IMPACTOMANUT IMPMANUT
                    ON SOLSERV.NRIMPACTOMANUT = IMPMANUT.NRIMPACTOMANUT
                LEFT JOIN 
                    FCM_GERENCIAMENTOCHK
                    ON SOLSERV.NRGERENCIAMENTOCHK = FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK
                LEFT JOIN 
                    FCM_CHECKLISTAGEND
                    ON SOLSERV.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN FCM_LOCALCLI
                    ON FCM_LOCALCLI.NRLOCALCLIENTE = SOLSERV.NRLOCALCLIENTE
                LEFT JOIN CLIENTE
                    ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                WHERE SOLSERV.NRSOLICITACAOSERV IS NOT NULL
                ". $filtroTipo;

                $sql .= "ORDER BY SOLSERV.DTSOLICITACAOSERV DESC";
    
    return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }
}