<?php

namespace FM\Service;

use FM\Factory\Fm00003000ChecklistFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\Fm00000000HabilidadesService;

class Fm00003000ChecklistService
{

    protected $ChecklistFactory;
    protected $FindService;
    protected $Environment;
    protected $newCode;
    protected $habilidadeService;
    protected $connection;

    public function __construct(EntityManager $entityManager, Fm00003000ChecklistFactory $ChecklistFactory, FindService $FindService, Environment $Environment, NewCode $newCode, Fm00000000HabilidadesService $Fm00000000HabilidadesService)
    {
        $this->ChecklistFactory          = $ChecklistFactory;
        $this->FindService               = $FindService;
        $this->connection                = $entityManager->getConnection();
        $this->Environment               = $Environment;
        $this->newCode                   = $newCode;
        $this->habilidadeService         = $Fm00000000HabilidadesService;
    }

    /* INICIO TAB DE CHECKLIST */
    public function salvarChecklist($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $nrchecklist = $this->newCode->getCodeByTableName("FCM_CHECKLIST", $NRORG, 1, 10, true);
            $cdOperador = $this->Environment->getCdOperador();

            $this->ChecklistFactory->salvarChecklist(array(
                "NRCHECKLIST"       => $nrchecklist,
                "NMCHECKLIST"       => $parametros['NMCHECKLIST'],
                "NRTIPOLOCALCLI"    => $parametros['NRTIPOLOCALCLI'],
                "NRORG"             => $NRORG,
                "CDOPERINCLUSAO"    => $cdOperador,
                "NRORGINCLUSAO"     => $NRORG,
                "IDATIVO"           => 'S'
            ));

            foreach ($parametros['NRATIVIDADE'] as $itens) {
                $nrAtividadeChk = $this->newCode->getCodeByTableName("FCM_ATIVIDADECHK", $NRORG, 1, 10, true);
                $this->ChecklistFactory->salvarItens(array(
                    "NRATIVIDADECHK"    => $nrAtividadeChk,
                    "NRATIVIDADE"       => $itens,
                    "NRCHECKLIST"       => $nrchecklist,
                    "NRORG"             => $NRORG,
                    "CDOPERINCLUSAO"    => $cdOperador,
                    "NRORGINCLUSAO"     => $NRORG,
                    "IDATIVO"           => 'S'
                ));

                foreach ($parametros['NRTAREFA'] as $atividade) {
                    $retorno = $this->ChecklistFactory->verificaExistenciaAtividade(array(
                        "NRATIVIDADE"   => $itens,
                        "NRTAREFA"      => $atividade
                    ));

                    $nrTarefaChk = $this->newCode->getCodeByTableName("FCM_TAREFACHK", $NRORG, 1, 10, true);
                    if ($retorno != 0) {
                        $this->ChecklistFactory->salvarAtividade(array(
                            "NRTAREFACHK"       => $nrTarefaChk,
                            "NRTAREFA"          => $atividade,
                            "NRCHECKLIST"       => $nrchecklist,
                            "NRATIVIDADECHK"    => $nrAtividadeChk,
                            "NRORG"             => $NRORG,
                            "CDOPERINCLUSAO"    => $cdOperador,
                            "NRORGINCLUSAO"     => $NRORG,
                            "IDATIVO"           => 'S'
                        ));

                        foreach ($parametros['CRITERIOS'] as $criterio) {
                            if ($criterio['NRTAREFA'] == $atividade) {
                                $this->ChecklistFactory->salvarCriterio(array(
                                    "NRCRITERIOTAREFACHK"   => $this->newCode->getCodeByTableName("FCM_CRITERIOTAREFACHK", $NRORG, 1, 10, true),
                                    "NRCRITERIOTAREFA"      => $criterio['NRCRITERIO'],
                                    "NRTAREFACHK"           => $nrTarefaChk,
                                    "NRORG"                 => $NRORG,
                                    "CDOPERINCLUSAO"        => $cdOperador,
                                    "NRORGINCLUSAO"         => $NRORG,
                                    "IDATIVO"               => 'S'
                                ));
                            }
                        }

                        foreach ($parametros['ESCOPOS'] as $escopo) {
                            if ($escopo['NRTAREFA'] == $atividade) {
                                $this->ChecklistFactory->salvarEscopo(array(
                                    "NRESCOPOSTAREFACHK"   => $this->newCode->getCodeByTableName("FCM_ESCOPOSTAREFACHK", $NRORG, 1, 10, true),
                                    "NRCHECKLIST"          => $nrchecklist,
                                    "NRTAREFACHK"          => $nrTarefaChk,
                                    "NRESCOPOSTAREFA"      => $escopo['NRESCOPO'],
                                    "NRORG"                => $NRORG,
                                    "CDOPERINCLUSAO"       => $cdOperador,
                                    "NRORGINCLUSAO"        => $NRORG,
                                    "IDATIVO"              => "S",
                                ));
                            }
                        }
                    }
                }
            }
            $this->connection->commit();
            return "ok";
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function edicaoChecklist($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();
            $tarefasAssociadas = array();
            $filtro = '';


            if ($parametros['IDATIVO'] !== 'S') {
                $params = array("NRCHECKLIST" => $parametros['NRCHECKLIST']);
                $filtro = " AND FCM_CHECKLISTAGEND.IDATIVO = 'S'";

                $tarefas = $this->ChecklistFactory->getTarefasCheck($params, $filtro);

                if (empty($tarefas)) {
                    $parametros['IDATIVO'] = 'N';
                } else {
                    $parametros['IDATIVO'] = 'S';
                    $tarefasAssociadas = $this->verificaDuplicidadeTarefasAssociadas($tarefas, $tarefasAssociadas);
                }
            } else {
                $parametros['IDATIVO'] = 'S';
            }

            $this->ChecklistFactory->editarChecklist(array(
                "NMCHECKLIST"   => $parametros['NMCHECKLIST'],
                "NRCHECKLIST"   => $parametros['NRCHECKLIST'],
                "IDATIVO"       => $parametros['IDATIVO'],
                "NRORGULTATU"   => $NRORG,
                "CDOPERULTATU"  => $cdOperador,
            ));

            $this->connection->commit();
            return $tarefasAssociadas;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getChecklist()
    {
        return $this->ChecklistFactory->getChecklist();
    }

    public function excluirChecklist($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $arrayNrTarefaChkExclusao = array();
            $arrayNrChecklistExclusao = array();
            $tarefasAssociadas = array();

            foreach ($parametros['NRS'] as $param) {
                $params = array("NRCHECKLIST" => $param['NRCHECKLIST']);
                $tarefas = $this->ChecklistFactory->getTarefasCheck($params, '');
            
                if (empty($tarefas)) {
                    array_push($arrayNrChecklistExclusao, $param['NRCHECKLIST']);
                    $retorno = $this->ChecklistFactory->getNrtarefachk($params);
                    
                    if (!empty($retorno)) {
                        foreach ($retorno as $value) {
                           array_push($arrayNrTarefaChkExclusao, $value['NRTAREFACHK']);
                        }
                    }
                } else {
                    $tarefasAssociadas = $this->verificaDuplicidadeTarefasAssociadas($tarefas, $tarefasAssociadas);
                }
            }

            if (!empty($arrayNrChecklistExclusao) && !empty($arrayNrTarefaChkExclusao)) {
                $nrTarefaChk = $this->FindService->formatarArray($arrayNrTarefaChkExclusao);
                $nrchecklist = $this->FindService->formatarArray($arrayNrChecklistExclusao);
                $this->ChecklistFactory->deleteChecklist($nrchecklist, $nrTarefaChk);
            }

            $this->connection->commit();
            return $tarefasAssociadas;
        } catch (\Exception $ex) {
            $this->connection->rollback();
            throw $ex;
        }
    }

    public function verificaDuplicidadeTarefasAssociadas($tarefas, $tarefasAssociados)
    {
        foreach ($tarefas as $value) {
            // Verifica diretamente dentro de $checklistAssociados
            $jaExiste = false;

            foreach ($tarefasAssociados as $associadas) {
                if ($associadas['NRCHECKLISTAGEND'] === $value['NRCHECKLISTAGEND']) {
                    $jaExiste = true;
                    break;
                }
            }

            // Insere a atividade apenas se não for duplicada
            if (!$jaExiste) {
                $tarefasAssociados[] = [
                    'NMCHECKLISTAGEND'  => $value['NMCHECKLISTAGEND'],
                    'NRCHECKLISTAGEND'  => $value['NRCHECKLISTAGEND'],
                    'IDSITUACAOAGEND'   => $value['IDSITUACAOAGEND'],
                    'DTHORAAGEND'       => $value['DTHORAAGEND'],
                    'IDATIVO'           => $value['IDATIVO'],
                    'CLIENTE'           => $value['CLIENTE'],
                    'NMLOCALCLIENTE'    => $value['NMLOCALCLIENTE'],
                ];
            }
        }

        return $tarefasAssociados;
    }

    public function getAtividadesChecklist($parametros)
    {
        $retorno = $this->FindService->formatarArray($parametros['NRATIVIDADE']);
        return $this->ChecklistFactory->getAtividadesChecklist($retorno);
    }

    public function getDadosChecklist($parametros)
    {

        $nrTarefa = $this->FindService->formatarArray($parametros['NRTAREFA']);
        $nratividade = $this->FindService->formatarArray($parametros['NRATIVIDADE']);

        return $this->ChecklistFactory->getDadosChecklist($nrTarefa, $nratividade);
    }
    
    
    public function getDadosChecklistEdicao($parametros)
    {
        return $this->ChecklistFactory->getDadosChecklistEdicao(array(
            "NRCHECKLIST"   => $parametros['NRCHECKLIST']
        ));
    }


    public function getCriteriosChecklist($parametros)
    {
        $filtro = "";
        if(isset($parametros['NRTAREFA'])) {
            $filtro .= "WHERE NRTAREFA IN (" . $parametros["NRTAREFA"] . ")";
        }
        return $this->ChecklistFactory->getCriteriosChecklist($filtro);
    }
    /* FIM TAB DE CHECKLIST */

    /* INICIO TAB DE ITENS */
    public function getItensChecklist()
    {
        return $this->ChecklistFactory->getItensChecklist();
    }

    // public function salvarNovoCriterio(array $row): array
    // {
    //     $NRORG = $this->Environment->getNrOrg();
    //     $cdOperador = $this->Environment->getCdOperador();

    //     try {
    //         $this->connection->beginTransaction();
    //         $NRCRITERIOTAREFA = $this->newCode->getCodeByTableName("FCM_CRITERIOTAREFA", $NRORG, 1, 10, true);

    //         $params = array(
    //             'NRCRITERIOTAREFA'  => $NRCRITERIOTAREFA,
    //             'DSCRITERIOTAREFA'  => $row['NOME'],
    //             'NRTAREFA'          => $row['ATIVIDADES'],
    //             'NRORG'             => $NRORG,
    //             'CDOPERINCLUSAO'    => $cdOperador,
    //             'NRORGINCLUSAO'     => $NRORG
    //         );
    //         $this->ChecklistFactory->inserirCriterio($params);
    //         $this->connection->commit();

    //         return $this->salvarRelacaoCriterioTarefaChk($params);
    //     } catch (\Throwable $th) {
    //         $this->connection->rollback();
    //         throw $th;
    //     }
    // }

    // private function salvarRelacaoCriterioTarefaChk(array $row): array
    // {
    //     try {
    //         $this->connection->beginTransaction();
    //         $NRCRITERIOTAREFACHK = $this->newCode->getCodeByTableName("FCM_CRITERIOTAREFACHK", $row["NRORG"], 1, 10, true);

    //         $params = array(
    //             'NRORG'                 => $row["NRORG"],
    //             'NRORGINCLUSAO'         => $row["NRORG"],
    //             'NRTAREFACHK'           => $row["NRTAREFA"],
    //             'CDOPERINCLUSAO'        => $row["CDOPERINCLUSAO"],
    //             'NRCRITERIOTAREFACHK'   => $NRCRITERIOTAREFACHK,
    //             'NRCRITERIOTAREFA'      => $row["NRCRITERIOTAREFA"],
    //             "IDATIVO"               => $row["IDATIVO"],
    //         );

    //         $this->ChecklistFactory->salvarCriterio($params);
    //         $this->connection->commit();
    //         return $params;
    //     } catch (\Throwable $th) {
    //         $this->connection->rollback();
    //         throw $th;
    //     }
    // }

    public function salvarItemChecklist($parametros)//em uso
    {
        $this->connection->beginTransaction();
        $NRORG = $this->Environment->getNrOrg();

        try {
            if ($parametros['NRATIVIDADE'] == null) {
                $params = array(
                    "NRATIVIDADE"       => $this->newCode->getCodeByTableName("FCM_ATIVIDADE", $NRORG, 1, 10, true),
                    "NMATIVIDADE"       => $parametros['NMATIVIDADE'],
                    "NRORG"             => $NRORG,
                    "CDOPERINCLUSAO"    => $this->Environment->getCdOperador(),
                    "NRORGINCLUSAO"     => $NRORG,
                    "DSCORIDATIVIDADE"  => $parametros['DSCORIDATIVIDADE']
                );

                $this->ChecklistFactory->inserirItemChecklist($params);
            } else {
                $params = array(
                    "NRATIVIDADE"       => $parametros['NRATIVIDADE'],
                    "CDOPERULTATU"      => $this->Environment->getCdOperador(),
                    "NRORGULTATU"       => $NRORG,
                    "DSCORIDATIVIDADE"  => $parametros['DSCORIDATIVIDADE']
                );
                $this->ChecklistFactory->updateCorItemChecklist($params);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inativarItem($parametros)//em uso
    {
        $this->connection->beginTransaction();
        
        try {
            $nrOrg = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();
            $filtro = '';
            $nratividade = intval($parametros['NRATIVIDADE']);

            $checklistAssociados = array();
            $param = array(
                "IDATIVO"           => ($parametros['ACAO']) ? "S" : "N",
                "NRORGULTATU"       => $nrOrg,
                "CDOPERULTATU"      => $cdOperador,
            );

            $filtro = " AND CHK.IDATIVO = 'S'";
            $podeInativar = false;
    
            // Se a ação for para inativar é necessário saber se há checklist ativo associado
            if (!$parametros['ACAO']) {
                // Verifica se há algum checklist ativo vínculado ao item
                $retornoChecklistAssociados = $this->ChecklistFactory->getChecklistsAssociadosItens(array(
                    "NRATIVIDADE" => $nratividade
                ), $filtro);

                /* Se não possuir nenhum checklist ativo, realiza a ação) */
                $podeInativar = (empty($retornoChecklistAssociados));
            } else {
                $podeInativar = true;
            }
            
            if ($podeInativar) {
                // Busca o itemChecklist associado ao Item (NRATIVIDADE)
                $numChecklistsItem = $this->ChecklistFactory->getItemChecklistAssociadoItem(array("NRATIVIDADE" => $nratividade));

                if (!empty($numChecklistsItem)) {
                    foreach ($numChecklistsItem as $relacItem) {
                        $param['NRITEMCHECKLIST'] = $relacItem['NRITEMCHECKLIST'];

                        $this->ChecklistFactory->updateEscoposItemChecklist($param);
                        $this->ChecklistFactory->updateCriteriosItemChecklist($param);
                        $this->ChecklistFactory->updateItemChecklist($param);

                        unset($param['NRITEMCHECKLIST']);
                    }
                }
                $param['NRATIVIDADE'] = $nratividade;
                $this->ChecklistFactory->updateItem($param);
            } else {
                // Retorna os checklist que tem itens associados sem duplicá-los
                $checklistAssociados = $this->verificaDuplicidadeChecklistsAssociados($retornoChecklistAssociados, $checklistAssociados);
            }

            $this->connection->commit();
            return $checklistAssociados;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function excluirItem($parametros)
    {
        $this->connection->beginTransaction();
        
        try {
            $checklistAssociados = array();
            $filtro = '';
            $nratividade = intval($parametros['NRATIVIDADE']);

            $retornoChecklistAssociados = $this->ChecklistFactory->getChecklistsAssociadosItens(array(
                "NRATIVIDADE" => $nratividade
            ), $filtro);
            
            if (empty($retornoChecklistAssociados)) {
                // Busca o itemChecklist associado ao Item (NRATIVIDADE)
                $numChecklistsItem = $this->ChecklistFactory->getItemChecklistAssociadoItem(array("NRATIVIDADE" => $nratividade));

                /*  Percorre os registros associados na seguinte ordem para exclusão:
                    Escopos: Exclui primeiro os registros na tabela Escopo, que são dependentes (filhos) da tabela ItemChecklist.
                    Critérios: Em seguida, exclui os registros na tabela Critérios, também dependentes (filhos) da tabela ItemChecklist.
                    ItensChecklist: Após remover as dependências, exclui os registros na própria tabela ItemChecklist.
                    Atividades: Por fim, remove Item na tabela Atividade, ao qual o itemChecklist estava associado.
                */
                if (!empty($numChecklistsItem)) {
                    foreach ($numChecklistsItem as $relacItem) {
                        $nrItem = array(
                            "NRITEMCHECKLIST" => $relacItem['NRITEMCHECKLIST'],
                        );

                        $this->ChecklistFactory->excluirItemChecklist($nrItem);
                    }
                }
                $this->ChecklistFactory->excluirItem(array('NRATIVIDADE' => $nratividade));
            } else {
                // Retorna os checklist que tem itens associados sem duplicá-los
                $checklistAssociados = $this->verificaDuplicidadeChecklistsAssociados($retornoChecklistAssociados, $checklistAssociados);
            }

            $this->connection->commit();
            return $checklistAssociados;
    
        } catch (\Exception $e) {
            $this->connection->rollback();
        }
    }    

    public function verificaDuplicidadeChecklistsAssociados($checklists, $checklistAssociados)
    {
        foreach ($checklists as $value) {
            // Verifica diretamente dentro de $checklistAssociados
            $jaExiste = false;
            foreach ($checklistAssociados as $checklist) {
                if ($checklist['NRCHECKLIST'] === $value['NRCHECKLIST']) {
                    $jaExiste = true;
                    break;
                }
            }

            // Insere a atividade apenas se não for duplicada
            if (!$jaExiste) {
                $checklistAssociados[] = [
                    'NMCHECKLIST' => $value['NMCHECKLIST'],
                    'NRCHECKLIST' => $value['NRCHECKLIST'],
                    'IDATIVO'     => $value['IDATIVO']
                ];
            }
        }

        return $checklistAssociados;
    }

    public function inserirItemChecklistCrit($nrItemChecklist, $criterios, $nrOrg, $cdOperador)
    {
        $this->connection->beginTransaction();
        try {
            foreach ($criterios as $criterio) {
                $this->ChecklistFactory->inserirCriterioItemChecklist(array(
                    "NRITEMCHECKLISTCRIT"   => $this->newCode->getCodeByTableName("FCM_ITEMCHECKLISTCRIT", $nrOrg, 1, 10, true),
                    "NRITEMCHECKLIST"       => $nrItemChecklist,
                    "NRCRITERIOTAREFA"      => $criterio,
                    "NRORG"                 => $nrOrg,
                    "CDOPERINCLUSAO"        => $cdOperador,
                    "NRORGINCLUSAO"         => $nrOrg,
                ));
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inserirItemChecklistEsc($nrItemChecklist, $escopos, $nrOrg, $cdOperador)
    {
        $this->connection->beginTransaction();
        try {
            foreach ($escopos as $escopo) {
                $this->ChecklistFactory->inserirEscopoItemChecklist(array(
                    "NRITEMCHECKLISTESC"    => $this->newCode->getCodeByTableName("FCM_ITEMCHECKLISTESC", $nrOrg, 1, 10, true),
                    "NRITEMCHECKLIST"       => $nrItemChecklist,
                    "NRESCOPOSTAREFA"       => $escopo,
                    "NRORG"                 => $nrOrg,
                    "CDOPERINCLUSAO"        => $cdOperador,
                    "NRORGINCLUSAO"         => $nrOrg,
            
                ));
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function updateItemChecklist($parametros, $nrOrg, $cdOperador)
    {
        $this->connection->beginTransaction();

        try {
            $this->ChecklistFactory->updateTarefaChecklist(array(
                "NRITEMCHECKLIST"   => $parametros['NRITEMCHECKLIST'],
                "NRTAREFA"          => $parametros['NRTAREFA'],
                "NRCRITERIOTAREFA"  => $parametros['NRCRITERIOTAREFA'],
                "IDATIVO"           => $parametros['IDATIVO'],
                "NRORGULTATU"       => $nrOrg,
                "CDOPERULTATU"      => $cdOperador,
                "NRESCOPOSTAREFA"   => $parametros['NRESCOPOSTAREFA'],
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function salvarRelacaoItemChecklist($parametros)
    {
        $this->connection->beginTransaction();
        
        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            foreach ($parametros['ROW'] as $row) {
                if($row['NRITEMCHECKLIST'] === 0) {
                    $nrItemChecklist = $this->newCode->getCodeByTableName("FCM_ITEMCHECKLIST", $NRORG, 1, 10, true);

                    // Insere na tabela de FCM_ITEMCHECKLIST
                    $this->ChecklistFactory->inserirRelacaoItemChecklist(array(
                        "NRITEMCHECKLIST"   => $nrItemChecklist,
                        "NRATIVIDADE"       => $parametros['NRATIVIDADE'],
                        "NRTAREFA"          => $row['NRTAREFA'],
                        "NRCRITERIOTAREFA"  => null,
                        "NRORG"             => $NRORG,
                        "CDOPERINCLUSAO"    => $cdOperador,
                        "NRORGINCLUSAO"     => $NRORG,
                        "NRESCOPOSTAREFA"   => null,
                    ));
        
                    // Insere os critérios na tabela de relação com a FCM_ITEMCHECKLIST
                    $this->inserirItemChecklistCrit($nrItemChecklist, $row['NRCRITERIOTAREFA'], $NRORG, $cdOperador);
        
                    // Insere os escopos na tabela de relação com a FCM_ITEMCHECKLIST
                    $this->inserirItemChecklistEsc($nrItemChecklist, $row['NRESCOPOSTAREFA'], $NRORG, $cdOperador);
                } else {
                    $checklistsAtivosAssociados = $this->inativarTarefasChecklist($parametros);
                }
            }
           
            $this->connection->commit();
            return $checklistsAtivosAssociados;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inativarTarefasChecklist($parametros)
    {
        $this->connection->beginTransaction();
        
        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();
            $filtro = '';
            $checklistsAssociados = [];

            foreach ($parametros['ROW'] as $itemChecklist) {
                $paramsUpdate = array(
                    "NRORGULTATU"       => $NRORG,
                    "CDOPERULTATU"      => $cdOperador,
                    "IDATIVO"           => null,
                    "NRITEMCHECKLIST"   => $itemChecklist['NRITEMCHECKLIST']
                );

                if ($itemChecklist['IDATIVO'] !== 'S') {
                    $criterios = $this->FindService->formatarArray($itemChecklist['NRCRITERIOTAREFA']);
                    $escopos = $this->FindService->formatarArray($itemChecklist['NRESCOPOSTAREFA']);
                    $filtro = "AND FCM_CHECKLIST.IDATIVO = 'S'";

                    $params = array(
                        "NRATIVIDADE"       => $parametros['NRATIVIDADE'],
                        "NRTAREFA"          => $itemChecklist['NRTAREFA'],
                        "NRITEMCHECKLIST"   => $itemChecklist['NRITEMCHECKLIST'],
                    );

                    $checklistsAtivos = $this->ChecklistFactory->getChecklistRelacionadoAtivos($params, $criterios, $escopos, $filtro);

                    if (empty($checklistsAtivos)) {
                        $paramsUpdate['IDATIVO'] = 'N';
                        $this->ChecklistFactory->updateItemChecklistEsc($paramsUpdate);
                        $this->ChecklistFactory->updateItemChecklistCrit($paramsUpdate);
                        $this->ChecklistFactory->updateItemChecklist($paramsUpdate);
                    } else {
                        $checklistsAssociados = $this->verificaDuplicidadeChecklistsAssociados($checklistsAtivos, $checklistsAssociados);
                    }
                } else {
                    $paramsUpdate['IDATIVO'] = 'S';
                        $this->ChecklistFactory->updateItemChecklistEsc($paramsUpdate);
                        $this->ChecklistFactory->updateItemChecklistCrit($paramsUpdate);
                        $this->ChecklistFactory->updateItemChecklist($paramsUpdate);
                }
            }

            $this->connection->commit();
            return $checklistsAssociados;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function excluirItemChecklist($parametros)
    {
        $this->connection->beginTransaction();
        
        try {
            $checklistsAssociados = [];
            $itensChecklistAExcluir = [];

            foreach ($parametros['ROW'] as $itemChecklist) {
                $criterios = $this->FindService->formatarArray($itemChecklist['NRCRITERIOTAREFA']);
                $escopos = $this->FindService->formatarArray($itemChecklist['NRESCOPOSTAREFA']);

                $params = array(
                    "NRATIVIDADE"       => $parametros['NRATIVIDADE'],
                    "NRTAREFA"          => $itemChecklist['NRTAREFA'],
                    "NRITEMCHECKLIST"   => $itemChecklist['NRITEMCHECKLIST'],
                );
                
                $checklistsAtivos = $this->ChecklistFactory->getChecklistRelacionadoAtivos($params, $criterios, $escopos, '');

                if (empty($checklistsAtivos)) {
                    array_push($itensChecklistAExcluir, $itemChecklist['NRITEMCHECKLIST']);
                } else {
                    $checklistsAssociados = $this->verificaDuplicidadeChecklistsAssociados($checklistsAtivos, $checklistsAssociados);
                }
            }

            if (!empty($itensChecklistAExcluir)) {
                $nrItens = $this->FindService->formatarArray($itensChecklistAExcluir);
                $this->ChecklistFactory->excluirItemChecklist($nrItens);
            }

            $this->connection->commit();
            return $checklistsAssociados;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    public function getTarefasChecklist($parametros)
    {
        $param = array(
            "NRATIVIDADE" => $parametros['NRATIVIDADE']
        );

        $dados = $this->ChecklistFactory->getTarefasChecklist($param);
        foreach ($dados['data'] as &$value) {
            $value['NRESCOPOSTAREFA'] = array_map('intval', $value['NRESCOPOSTAREFA']);
            $value['NRCRITERIOTAREFA'] = array_map('intval', $value['NRCRITERIOTAREFA']);
        }

        return $dados;
    }

    public function verificarNomeItens(array $params): array
    {
        try {
            $NRORG = $this->Environment->getNrOrg();

            $params = array(
                "NRORG"         => $NRORG,
                "NOME"          => $params['NOME'],
            );
            return $this->ChecklistFactory->verificarNomeItens($params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function verificaDuplicidadeItensAssociados($itens, $itensAssociados)
    {
        foreach ($itens as $value) {
            // Verifica diretamente dentro de $itensAssociados
            $jaExiste = false;
            foreach ($itensAssociados as $atividade) {
                if ($atividade['NRATIVIDADE'] === $value['NRATIVIDADE']) {
                    $jaExiste = true;
                    break;
                }
            }

            // Insere a atividade apenas se não for duplicada
            if (!$jaExiste) {
                $itensAssociados[] = [
                    'NMATIVIDADE' => $value['NMATIVIDADE'],
                    'NRATIVIDADE' => $value['NRATIVIDADE']
                ];
            }
        }

        return $itensAssociados;
    }
    /* FIM TAB DE ITENS */
    
    /* INICIO TAB ATIVIDADES E CRITÉRIOS */
    public function getAtividadesCriterios()
    {
        return $this->ChecklistFactory->getAtividadesCriterios();
    }

    public function getAtividadesAtivas()
    {
        return $this->ChecklistFactory->getAtividadesAtivas();
    }

    public function salvarAtividadeCriterio($parametros)
    {
        $this->connection->beginTransaction();
        
        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();
            $itensAssociados = array();
        
            if ($parametros['NRROW'] == null) {
                if ($parametros['TIPOADICAO'] ==  'Critério') {
                    $params = array(
                        'NRCRITERIOTAREFA'  => $this->newCode->getCodeByTableName("FCM_CRITERIOTAREFA", $NRORG, 1, 10, true),
                        'DSCRITERIOTAREFA'  => $parametros['NOME'],
                        'NRTAREFA'          => $parametros['ATIVIDADES'],
                        'NRORG'             => $NRORG,
                        'CDOPERINCLUSAO'    => $cdOperador,
                        'NRORGINCLUSAO'     => $NRORG,
                        "IDATIVO"               => $parametros["IDATIVO"]
                    );
                    $this->ChecklistFactory->inserirCriterio($params);
                } elseif ($parametros['TIPOADICAO'] ==  'Atividade') {
                    $params = array(
                        'NRTAREFA'              => $this->newCode->getCodeByTableName("FCM_TAREFA", $NRORG, 1, 10, true),
                        'NMTAREFA'              => $parametros['NOME'],
                        'NRORG'                 => $NRORG,
                        'CDOPERINCLUSAO'        => $cdOperador,
                        'NRORGINCLUSAO'         => $NRORG,
                        'NRPRIORIDADETAREFA'    => $parametros['PRIORIDADE'],
                        "IDATIVO"               => $parametros["IDATIVO"]
                    );
                    $this->ChecklistFactory->inserirAtividade($params);

                    foreach ($parametros["HABILIDADES"] as $row) {
                        $this->habilidadeService->relacionarAtividadeHabilidade(array_merge($params, ["NRHABILIDADE" => $row]));
                    }
                }
            } else if ($parametros["IDATIVO"] === 'S') {
                if ($parametros['TIPOADICAO'] ==  'Critério') {
                    $nrcriterio = explode('C', $parametros['NRROW']);

                    $params = array(
                        'NRCRITERIOTAREFA'  => $nrcriterio[1],
                        'DSCRITERIOTAREFA'  => $parametros['NOME'],
                        'CDOPERULTATU'      => $cdOperador,
                        'NRORGULTATU'       => $NRORG,
                        "IDATIVO"           => $parametros["IDATIVO"]
                    );
                    $this->ChecklistFactory->updateCriterio($params);
                } else if ($parametros['TIPOADICAO'] ==  'Atividade') {
                    $nratividade = explode('T', $parametros['NRROW']);

                    $params = array(
                        'NRTAREFA'              => $nratividade[1],
                        'NMTAREFA'              => $parametros['NOME'],
                        'CDOPERULTATU'          => $cdOperador,
                        'NRORGULTATU'           => $NRORG,
                        'NRPRIORIDADETAREFA'    => $parametros['PRIORIDADE'],
                        "IDATIVO"               => $parametros["IDATIVO"],
                    );
                    $this->ChecklistFactory->updateAtividade($params);

                    $this->habilidadeService->excluirAtividadeHabilidades($parametros);

                    foreach ($parametros["HABILIDADES"] as $row) {
                        $this->habilidadeService->relacionarAtividadeHabilidade(array_merge($params, ["NRHABILIDADE" => $row]));
                    }
                }
            } else {
                if ($parametros['TIPOADICAO'] ==  'Critério') {
                    $nrcriterio = explode('C', $parametros['NRROW']);
                    $retornoCriterios = $this->ChecklistFactory->getItensAssociadosCriterios(array(
                        'NRCRITERIOTAREFA'=> $nrcriterio[1],
                    ));

                    if (empty($retornoCriterios)) {
                        $this->ChecklistFactory->updateCriterio(array(
                            'NRCRITERIOTAREFA'  => $nrcriterio[1],
                            'DSCRITERIOTAREFA'  => $parametros['NOME'],
                            'CDOPERULTATU'      => $cdOperador,
                            'NRORGULTATU'       => $NRORG,
                            "IDATIVO"           => $parametros["IDATIVO"]
                        ));
                    } else {
                        // Retorna os itens que tem critérios associados sem duplicá-los
                        $itensAssociados = $this->verificaDuplicidadeItensAssociados($retornoCriterios, $itensAssociados);
                    }
                } else if ($parametros['TIPOADICAO'] ==  'Atividade') {
                    $nratividade = explode('T', $parametros['NRROW']);
                    $retornoAtividades = $this->ChecklistFactory->getItensAssociadosAtividades(array(
                        'NRTAREFA' => $nratividade[1]));

                    if (empty($retornoAtividades)) {
                        $params = array(
                            'NRTAREFA'              => $nratividade[1],
                            'NMTAREFA'              => $parametros['NOME'],
                            'CDOPERULTATU'          => $cdOperador,
                            'NRORGULTATU'           => $NRORG,
                            'NRPRIORIDADETAREFA'    => $parametros['PRIORIDADE'],
                            "IDATIVO"               => $parametros["IDATIVO"],
                        );
                        $this->ChecklistFactory->updateAtividade($params);
    
                        $this->habilidadeService->excluirAtividadeHabilidades($parametros);
    
                        foreach ($parametros["HABILIDADES"] as $row) {
                            $this->habilidadeService->relacionarAtividadeHabilidade(array_merge($params, ["NRHABILIDADE" => $row]));
                        }
                    } else {
                        // Retorna os itens que tem atividades associados sem duplicá-los
                        $itensAssociados = $this->verificaDuplicidadeItensAssociados($retornoAtividades, $itensAssociados);
                    }
                }
            }

            $this->connection->commit();
            return $itensAssociados;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inativarAtividadesCriterios($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $itensAssociados = array();
            $atividadesLiberadasExclusao =  array();
            $criteriosLiberadosExclusao = array();

            foreach ($parametros['NRS'] as $param) {
                if ($param['TIPO'] == 'Critério') {
                    // Busca os Itens que estão associados aos critérios
                    $retornoCriterios = $this->ChecklistFactory->getItensAssociadosCriterios(array(
                            'NRCRITERIOTAREFA' => $param['NR']));

                    if (empty($retornoCriterios)) {
                        // Já separa os critérios que estão liberados para serem excluídos
                        array_push($criteriosLiberadosExclusao, $param['NR']);
                    } else {
                        // Retorna os itens que tem critérios associados sem duplicá-los
                        $itensAssociados = $this->verificaDuplicidadeItensAssociados($retornoCriterios, $itensAssociados);
                    }
                } elseif ($param['TIPO'] == 'Atividade') {
                    // Busca os Itens que estão associados as atividades
                    $retornoAtividades = $this->ChecklistFactory->getItensAssociadosAtividades(array(
                        'NRTAREFA' => $param['NR']));

                    if (empty($retornoAtividades)) {
                        // Já separa as atividades que estão liberados para serem excluídas
                        array_push($atividadesLiberadasExclusao, $param['NR']);
                    } else {
                        // Retorna os itens que tem atividades associados sem duplicá-los
                        $itensAssociados = $this->verificaDuplicidadeItensAssociados($retornoAtividades, $itensAssociados);
                    }
                }
            }

            // Formata os critérios e atividades liberados para serem excluídos todos de uma vez
            $criteriosLiberadosExclusao = $this->FindService->formatarArray($criteriosLiberadosExclusao);
            $atividadesLiberadasExclusao = $this->FindService->formatarArray($atividadesLiberadasExclusao);

            if (!empty($criteriosLiberadosExclusao)) { 
                $this->ChecklistFactory->excluirCriterio($criteriosLiberadosExclusao);
            }

            if (!empty($atividadesLiberadasExclusao)) { 
                $this->ChecklistFactory->excluirAtividades($atividadesLiberadasExclusao);
            }

            $this->connection->commit();
            return $itensAssociados;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e; 
        }
    }

    /* FIM TAB ATIVIDADES E CRITÉRIOS */

    /* INICIO TAB DE ESCOPOS */
    public function getEscoposServicos() 
    {
        return $this->ChecklistFactory->getEscoposServicos();
    }

    public function salvarEscopoServico($params) 
    {
        $this->connection->beginTransaction();
        $NRORG = $this->Environment->getNrOrg();
        $operador = $this->Environment->getCdOperador();

        try {
            $atividadesAssociadas = array();

            foreach ($params as $row) {
                if ($row["NRESCOPOSTAREFA"] == 0) {
                    $param = array(
                        "NRESCOPOSTAREFA"   => $this->newCode->getCodeByTableName("FCM_ESCOPOSTAREFA", $NRORG, 1, 10, true),
                        "NMESCOPOSTAREFA"   => $row["NMESCOPOSTAREFA"],
                        "IDATIVO"           => $row["IDATIVO"],
                        "NRORG"             => $NRORG,
                        "NRORGINCLUSAO"     => $NRORG,
                        "CDOPERINCLUSAO"    => $operador
                    );

                    $this->ChecklistFactory->inserirEscopoServico($param);
                } else if($row["IDATIVO"] === 'S') {
                    $param = array(
                        "NRESCOPOSTAREFA"   => $row["NRESCOPOSTAREFA"],
                        "NMESCOPOSTAREFA"   => $row["NMESCOPOSTAREFA"],
                        "IDATIVO"           => $row["IDATIVO"],
                        "NRORG"             => $NRORG,
                        "CDOPERULTATU"      => $operador,
                        "NRORGINCLUSAO"     => $NRORG,
                        "CDOPERINCLUSAO"    => $operador,
                        "NRORGULTATU"       => $NRORG
                    );

                    $this->ChecklistFactory->updateEscopoServico($param);
                } else {
                    $filtro = ` AND FCM_ITEMCHECKLIST.IDATIVO = 'S' AND FCM_ATIVIDADE.IDATIVO = 'S'`;
                    $retorno = $this->ChecklistFactory->getEscoposUtil($row, $filtro);

                    if (empty($retorno)) {
                        // Já atualizaa os escopos que estão liberados para serem inativados
                        $this->ChecklistFactory->updateEscopoServico(array(
                            "NRESCOPOSTAREFA"   => $row["NRESCOPOSTAREFA"],
                            "NMESCOPOSTAREFA"   => $row["NMESCOPOSTAREFA"],
                            "IDATIVO"           => $row["IDATIVO"],
                            "NRORG"             => $NRORG,
                            "CDOPERULTATU"      => $operador,
                            "NRORGINCLUSAO"     => $NRORG,
                            "CDOPERINCLUSAO"    => $operador,
                            "NRORGULTATU"       => $NRORG
                        ));
                        $this->connection->commit();
                    } else {
                        foreach ($retorno as $value) {
                            // Verifica diretamente dentro de $atividadesAssociadas
                            $jaExiste = false;
                            foreach ($atividadesAssociadas as $atividade) {
                                if ($atividade['NRATIVIDADE'] === $value['NRATIVIDADE'] && $atividade['NRTAREFA'] === $value['NRTAREFA']) {
                                    $jaExiste = true;
                                    break;
                                }
                            }
                
                            // Insere a atividade apenas se não for duplicada
                            if (!$jaExiste) {
                                $atividadesAssociadas[] = [
                                    'NMATIVIDADE'   => $value['NMATIVIDADE'],
                                    'NRATIVIDADE'   => $value['NRATIVIDADE'],
                                    'NMTAREFA'      => $value['NMTAREFA'],
                                    'NRTAREFA'      => $value['NRTAREFA']
                                ];
                            }
                        }
                    }
                }
            }
            
            return $atividadesAssociadas; 
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function excluirEscopos($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $itensAssociados = array();
            $escoposLiberadosExclusao = array();
        
            foreach ($parametros['NRS'] as $row) {
                // Busca os Itens e Atividades cujo Escopo está associado
                $retorno = $this->ChecklistFactory->getEscoposUtil($row, '');
                if (empty($retorno)) {
                    // Já separa os escopos que estão liberados para serem excluídos
                    array_push($escoposLiberadosExclusao, $row['NRESCOPOSTAREFA']);
                } else {
                    foreach ($retorno as $value) {
                        // Verifica diretamente dentro de $itensAssociados
                        $jaExiste = false;
                        foreach ($itensAssociados as $atividade) {
                            if ($atividade['NRATIVIDADE'] === $value['NRATIVIDADE']) {
                                $jaExiste = true;
                                break;
                            }
                        }
            
                        // Insere a atividade apenas se não for duplicada
                        if (!$jaExiste) {
                            $itensAssociados[] = [
                                'NMATIVIDADE' => $value['NMATIVIDADE'],
                                'NRATIVIDADE' => $value['NRATIVIDADE']
                            ];
                        }
                    }
                }
            }
            
            if (!empty($escoposLiberadosExclusao)) { 
                $escoposLiberadosExclusao = $this->FindService->formatarArray($escoposLiberadosExclusao);
                $this->ChecklistFactory->excluirEscopos($escoposLiberadosExclusao);
            }
                    
            $this->connection->commit();
            return $itensAssociados;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    /* FIM TAB DE ESCOPOS */
}
