<?php

namespace FM\Service;

use FM\Factory\FmtSolicitacaoServFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FileServerService;

class FmtSolicitacaoServService
{
    protected $SolicitacaoServFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;
    protected $FileServerService;
    protected $fileServerUrl;

    public function __construct(FmtSolicitacaoServFactory $SolicitacaoServFactory, Environment $Environment, NewCode $newCode, EntityManager $entityManager, FileServerService $FileServerService, $fileServerUrl)
    {
        $this->SolicitacaoServFactory   = $SolicitacaoServFactory;
        $this->Environment              = $Environment;
        $this->newCode                  = $newCode;
        $this->connection               = $entityManager->getConnection();
        $this->FileServerService        = $FileServerService;
        $this->fileServerUrl            = $fileServerUrl;
    }

    public function getSolicitacoesServicos()
    {
        return $this->SolicitacaoServFactory->getSolicitacoesServicos();
    }

    public function saveRecusarSolicitacoes($parametros)
    {
        try {
            $this->connection->beginTransaction();

            foreach ($parametros["ROWS"] as $row) {
                $row["DSCOMENTARIO"] = $parametros["DSCOMENTARIO"];
                $this->recusarSolicitacao($row);
                $this->insertComentarioSolicitacao($row);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function recusarSolicitacao($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $parametroRecusar = array(
                "NRSOLICITACAOSERV" => $parametros["NRSOLICITACAOSERV"]
            );

            $this->SolicitacaoServFactory->recusarSolicitacao($parametroRecusar);

            $this->insertHistorico(array(
                "NRSOLICITACAOSERV" => $parametros["NRSOLICITACAOSERV"],
                "DSORDEMSERVHISTORICO" => "Solicitação de serviço recusada. Motivo: " . $parametros["DSCOMENTARIO"],
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function insertComentarioSolicitacao($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $cdOperador = $this->Environment->getCdOperador();
            $nrorg = $this->Environment->getNrorg();
            $NRCOMENTSOLICITSERV = $this->newCode->getCodeByTableName("FCM_COMENTSOLICITSERV", 20, 1, 10, true);

            $parametroComentario = array(
                "NRCOMENTSOLICITSERV"   => $NRCOMENTSOLICITSERV,
                "NRSOLICITACAOSERV"     => $parametros["NRSOLICITACAOSERV"],
                "DSCOMENTARIO"          => $parametros["DSCOMENTARIO"],
                "NRORG"                 => $nrorg,
                "CDOPERINCLUSAO"        => $cdOperador,
                "NRORGINCLUSAO"         => $nrorg,
                "CDOPERULTATU"          => $cdOperador,
                "NRORGULTATU"           => $nrorg,
                "CDOPERCOMENTARIO"      => $cdOperador
            );

            $this->SolicitacaoServFactory->insertComentarioSolicitacao($parametroComentario);

            $this->insertHistorico(array(
                "NRSOLICITACAOSERV"    => $parametros["NRSOLICITACAOSERV"],
                "DSORDEMSERVHISTORICO" => "Comentário adicionado à solicitação de serviço.",
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveSolicitacaoServico($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $cdOperador = $this->Environment->getCdOperador();
            $nrorg = $this->Environment->getNrorg();

            $NRSOLICITACAOSERV = $this->newCode->getCodeByTableName("FCM_SOLICITACAOSERV", 20, 1, 10, true);
            $parametroSolicitacao = array(
                "NRSOLICITACAOSERV"     => $NRSOLICITACAOSERV,
                "NREQUIPAMENTO"         => $parametros["NREQUIPAMENTO"],
                "NRCATEGORIAMANUT"      => $parametros["NRCATEGORIAMANUT"],
                "NRIMPACTOMANUT"        => $parametros["NRIMPACTOMANUT"],
                "DSSOLICITACAOSERV"     => $parametros["DSSOLICITACAOSERV"],
                "NRORG"                 => $nrorg,
                "CDOPERINCLUSAO"        => $cdOperador,
                "NRORGINCLUSAO"         => $nrorg,
                "CDOPERULTATU"          => $cdOperador,
                "NRORGULTATU"           => $nrorg,
                "NRGERENCIAMENTOCHK"    => (!empty($parametros["NRGERENCIAMENTOCHK"])) ? $parametros["NRGERENCIAMENTOCHK"] : null,
                "NRCHECKLISTAGEND"      => (!empty($parametros["NRCHECKLISTAGEND"])) ? $parametros["NRCHECKLISTAGEND"] : null,
                "NMSOLICITACAOSERV"     => $parametros["NMSOLICITACAOSERV"],
                "CDOPERSOLICITANTE"     => $parametros["CDOPERSOLICITANTE"],
                "NRLOCALCLIENTE"        => $parametros["NRLOCALCLIENTE"]
            );

            $this->SolicitacaoServFactory->saveSolicitacaoServico($parametroSolicitacao);

            if (!empty($parametros["FOTOS"])) {

                $parametroFoto = array(
                    "NRSOLICITACAOSERV" => $NRSOLICITACAOSERV,
                    "FOTOS"             => $parametros["FOTOS"]
                );
                $this->insertFotosSolicitacao($parametroFoto);
            }

            $parametros["NRSOLICITACAOSERV"] = $NRSOLICITACAOSERV;
            $this->insertHistoricoVistoria($parametros);

            $this->insertHistorico(array(
                "NRSOLICITACAOSERV" => $NRSOLICITACAOSERV,
                "DSORDEMSERVHISTORICO"  => 'Foi gerada uma nova solicitação de serviço.',
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function insertHistorico($param)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrorg();

            $this->SolicitacaoServFactory->insertHistorico(array(
                "NRORDEMSERVHISTORICO" => $this->newCode->getCodeByTableName("FCM_ORDEMSERVHISTORICO", $nrorg, 1, 10, true),
                "NRSOLICITACAOSERV"    => $param["NRSOLICITACAOSERV"] ?? null,
                "NRORDEMSERV"          => $param["NRORDEMSERV"] ?? null,
                "DSORDEMSERVHISTORICO" => $param["DSORDEMSERVHISTORICO"],
                "NRORG"                => $nrorg,
                "CDOPERINCLUSAO"       => $this->Environment->getCdOperador(),
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function insertFotosSolicitacao($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();

            foreach ($parametros["FOTOS"] as $foto) {
                $img = $this->FileServerService->uploadFoto([$foto], "foto_vistoria_fm");

                $NRIMGSOLICITACAOSERV = $this->newCode->getCodeByTableName('FCM_IMGSOLICITACAOSERV', 20, 1, 10, true);
                $parametroFoto = array(
                    "NRIMGSOLICITACAOSERV"  => $NRIMGSOLICITACAOSERV,
                    "NRSOLICITACAOSERV"     => $parametros["NRSOLICITACAOSERV"],
                    "DSCAMINHO"             => $this->fileServerUrl . "/files/" . $img["path"],
                    "NRORG"                 => $nrorg,
                    "DSTYPE"                => $img["type"],
                    "NRSIZE"                => $img["size"]
                );

                $this->SolicitacaoServFactory->insertFotoSolicitacao($parametroFoto);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
        }
    }

    public function getImpactoManut()
    {
        return $this->SolicitacaoServFactory->getImpactoManut();
    }

    public function getPrioridadesManut()
    {
        return $this->SolicitacaoServFactory->getPrioridadesManut();
    }

    public function getFotosSolicitacao($parametros)
    {
        $parametroFoto = array(
            "NRSOLICITACAOSERV" => $parametros["NRSOLICITACAOSERV"]
        );

        return $this->SolicitacaoServFactory->getFotosSolicitacao($parametroFoto);
    }

    public function getComentariosSolicitacao($parametros)
    {
        $parametroFoto = array(
            "NRSOLICITACAOSERV" => $parametros["NRSOLICITACAOSERV"]
        );

        return $this->SolicitacaoServFactory->getComentariosSolicitacao($parametroFoto);
    }

    public function getOperadorLogado()
    {
        return array(
            "CDOPERADOR" => $this->Environment->getCdOperador()
        );
    }

    public function getOperadores()
    {
        return $this->SolicitacaoServFactory->getOperadores();
    }

    public function getFuncionarios()
    {
        return $this->SolicitacaoServFactory->getFuncionarios();
    }

    public function getFuncionarioLogado()
    {
        $parametroFuncionario = array(
            "CDOPERADOR" => $this->Environment->getCdOperador()
        );

        return $this->SolicitacaoServFactory->getFuncionarioLogado($parametroFuncionario);
    }
    public function insertHistoricoVistoria($parametros)
    {
        try {
            $NRORG = $this->Environment->getNrOrg();
            $operadorLogado = $this->Environment->getCdOperador();

            if(isset($parametros["NRCHECKLISTAGEND"])) {
                $nmchecklistagend = $this->SolicitacaoServFactory->getNmchecklist($parametros);

                $this->SolicitacaoServFactory->insertHistoricoTarefa(array(
                    "NRCHKAGENDHISTORICO"       => $this->newCode->getCodeByTableName("FCM_CHKAGENDHISTORICO", $NRORG, 1, 10, true),
                    "NRCHECKLISTAGEND"          => $parametros["NRCHECKLISTAGEND"],
                    "NMCHECKLISTAGEND"          => $nmchecklistagend[0]["NMCHECKLISTAGEND"],
                    "DSGERENCIAMENTOCHK"        => "Foi gerada uma solicitação de serviço a partir dessa tarefa. Motivo: " . $parametros['DSSOLICITACAOSERV'],
                    "IDTIPOHISTORICO"           => 1,
                    "NRORG"                     => $NRORG,
                    "CDOPERINCLUSAO"            => $operadorLogado,
                    "NRORGINCLUSAO"             => $NRORG,
                    "CDOPERULTATU"              => $operadorLogado,
                    "NRORGULTATU"               => $NRORG
                ));

                $this->insertHistorico(array(
                    "NRSOLICITACAOSERV"    => $parametros["NRSOLICITACAOSERV"],
                    "DSORDEMSERVHISTORICO" => "Solicitação de serviço criada a partir de uma tarefa. Tarefa: " . $nmchecklistagend[0]["NMCHECKLISTAGEND"],
                ));
            }

            if(isset($parametros["NRGERENCIAMENTOCHK"])) {
                $nmgerenciamentochk = $this->SolicitacaoServFactory->getNmgerenciamento($parametros);

                $this->SolicitacaoServFactory->insertHistoricoVistoria(array(
                    "NRGERCHKHISTORICO"         => $this->newCode->getCodeByTableName("FCM_GERCHKHISTORICO", $NRORG, 1, 10, true),
                    "NRGERENCIAMENTOCHK"        => $parametros["NRGERENCIAMENTOCHK"],
                    "NMGERENCIAMENTOCHK"        => $nmgerenciamentochk[0]["NMGERENCIAMENTOCHK"],
                    "DSGERENCIAMENTOCHK"        => "Foi gerada uma solicitação de serviço a partir dessa vistoria. Motivo: " . $parametros['DSSOLICITACAOSERV'],
                    "IDTIPOHISTORICO"           => 1,
                    "NRORG"                     => $NRORG,
                    "CDOPERINCLUSAO"            => $operadorLogado,
                    "NRORGINCLUSAO"             => $NRORG,
                    "CDOPERULTATU"              => $operadorLogado,
                    "NRORGULTATU"               => $NRORG
                ));

                $this->insertHistorico(array(
                    "NRSOLICITACAOSERV"    => $parametros["NRSOLICITACAOSERV"],
                    "DSORDEMSERVHISTORICO" => "Solicitação de serviço criada a partir de uma vistoria. Vistoria: " . $nmgerenciamentochk[0]["NMGERENCIAMENTOCHK"],
                ));
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    public function getHistoricoSolServ($parametros)
    {
        return $this->SolicitacaoServFactory->getHistoricoSolServ($parametros);
    }

    public function getTecnicos($parametros)
    {
        $local = '';

        if (!empty($parametros['NRLOCALCLIENTE'])) {
            $local .= "'" . $parametros['NRLOCALCLIENTE'] . "'";
            return $this->SolicitacaoServFactory->getTecnicos($local);
        }
    }

    public function getEquipeTecnicos($parametros)
    {
        $local = '';

        if (!empty($parametros['NRLOCALCLIENTE'])) {
            $local .= "'" . $parametros['NRLOCALCLIENTE'] . "'";
            return $this->SolicitacaoServFactory->getEquipeTecnicos($local);
        }
    }
}
