import { Utilities } from '@/util/Utilities';
import {
    DialogService, Form, LoadingService, Modal, ModalService, Select, Text, TextInput,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, MemoryDatasource, Metadata,
} from '@zeedhi/core';
import {
    IterableComponentRender, TekGrid,
} from '@zeedhi/teknisa-components-common';
import ColorPicker from '@/components/colorPicker/ZdColorPicker';
import { TekIterableComponentRender } from '@zeedhi/teknisa-components-vuetify';

export class Fm00003002ItensController {
    public modalItens!: Modal;

    public modalTarefasChecklist!: Modal;

    public codGridEditavel: String = '';

    public tituloModal: String = 'Adicionar Itens';

    public ultAtividadeSelecionada: String = '';

    public ultAtividadeIsAtivo!: Boolean;

    public modalErroInativarItem!: Modal;

    public async criarModais() {
        this.modalErroInativarItem = await ModalService.createFromJson('fm_00003_000_checklist/modal/fm_00003_007_modal_inatividade_itens', true);
    }

    public async abrirModalErroInatividade() {
        this.modalErroInativarItem.show();
    }

    public fecharModalInatividade() {
        this.modalErroInativarItem.hide();
    }

    public destruirModais() {
        this.modalErroInativarItem.destroy();
    }

    public async abrirModalItens({ component }: any) {
        this.modalItens = await ModalService.createFromJsonPost('', { jsonPath: '/fm_00003_000_checklist/modal/fm_00003_002_salvar_itens' });
        await this.modalItens.show();
        const inputName = Metadata.getInstance<TextInput>('nomeTarefa');

        if (component.label === 'Alterar Cor') {
            inputName.isVisible = false;
            inputName.clearValidations();
            this.tituloModal = 'Alterar Cor';
            /* eslint-disable */
            Metadata.getInstance<TextInput>('nratividade').value = component.name.split('_')[1];
            /* eslint-enable */
        } else {
            inputName.isVisible = true;
            inputName.addValidation('required', {});
            this.tituloModal = 'Adicionar Itens';
        }
    }

    public fecharModalItens() {
        this.modalItens.hide();
        this.modalItens?.destroy();
    }

    public dialogInativarItem({ event, component }: any) {
        /* eslint-disable */
        this.ultAtividadeSelecionada = component.name.split('_')[1];
        /* eslint-enable */
        this.ultAtividadeIsAtivo = (component.label === 'Ativar');
        Utilities.showConfirmDeleteMessage(event, 'Fm00003002ItensController.inativarItem', `${component.label} Registro`, `Deseja realmente ${component.label} o registro selecionado?`);
    }

    public inativarItem() {
        const cards = Metadata.getInstance<TekIterableComponentRender>('card-iterable-itens');
        Http.post('/inativarItem', {
            NRATIVIDADE: this.ultAtividadeSelecionada,
            ACAO: this.ultAtividadeIsAtivo,
        }).then((response: any) => {
            if (response.data.length > 0) {
                this.montarModalErroInatividade(response.data, false);
                DialogService.hide();
            } else {
                DialogService.hide();
                Utilities.showSuccessMessage();
                cards.datasource.get();
            }
        });
    }

    public async montarModalErroInatividade(tarefas: any, exclusao = false) {
        await this.abrirModalErroInatividade();
        const grid = Metadata.getInstance<TekGrid>('grid-tarefas-conflito-inatividade-itens');
        const campoTextErro = Metadata.getInstance<Text>('texto-erro-itens');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        if (exclusao) {
            campoTextErro.text = '<strong>Não é possivel excluir o item selecionado por quê há checklist(s) associado(s) a ele.</strong>';
            grid.getColumn('IDATIVO').isVisible = true;
        } else {
            campoTextErro.text = '<strong>Não é possivel inativar o item selecionado por quê há checklist(s) ativo(s) associado(s) a ele.</strong>';
            grid.getColumn('IDATIVO').isVisible = false;
        }

        tarefas.forEach((element: any) => {
            const dados: any = {
                NRCHECKLIST: element.NRCHECKLIST,
                NMCHECKLIST: element.NMCHECKLIST,
                IDATIVO: element.IDATIVO,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }

    public dialogExcluirItem({ event, component }: any) {
        /* eslint-disable */
        this.ultAtividadeSelecionada = component.name.split('_')[1];
        /* eslint-enable */
        Utilities.showConfirmDeleteMessage(event, 'Fm00003002ItensController.excluirItens', 'Excluir Registros', 'Deseja realmente excluir o registro selecionado?');
    }

    public excluirItens() {
        Http.post('/excluirItens', {
            NRATIVIDADE: this.ultAtividadeSelecionada,
        }).then((response: any) => {
            if (response.data.length > 0) {
                this.montarModalErroInatividade(response.data, true);
            } else {
                const cards = Metadata.getInstance<TekIterableComponentRender>('card-iterable-itens');
                Utilities.showSuccessMessage();
                cards.datasource.get();
            }
            DialogService.hide();
        });
    }

    public verificarNomeItensExistente({ component }: any) {
        Http.post('/verificarNomeItens', {
            NOME: component.value,
        }).then((response: any) => {
            if (response.data.data.length !== 0) {
                Utilities.showErrorMessage('Já existe um item com esse nome!');
                component.value = null;
            }
        });
    }

    public async abrirModalTarefasChecklist() {
        this.modalTarefasChecklist = await ModalService.createFromJsonPost('', { jsonPath: '/fm_00003_000_checklist/modal/fm_00003_002_salvar_tarefas_checklist' });
        await this.modalTarefasChecklist.show();
    }

    public fecharModalTarefasChecklist() {
        this.modalTarefasChecklist.hide();
        this.modalTarefasChecklist?.destroy();
    }

    public salvarItens() {
        const form = Metadata.getInstance<Form>('form-itens');
        const cards = Metadata.getInstance<IterableComponentRender>('card-iterable-itens');

        if (form.validate()) {
            Utilities.save('/salvarItemChecklist', {
                NRATIVIDADE: form.value.nratividade,
                NMATIVIDADE: form.value.nomeTarefa,
                DSCORIDATIVIDADE: Metadata.getInstance<ColorPicker>('COLORPICKER').value,
            }).then(() => {
                Utilities.showSuccessMessage();
                cards.datasource.get();
                this.fecharModalItens();
            });
        } else {
            Utilities.alertMessage('Preencha todos os campos obrigatórios (*)!', 'error');
        }
    }

    public adicionarNovaLinha({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRITEMCHECKLIST: newId }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NRTAREFA_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public changeCriterios({ component }: any) {
        const selectCriterio = Metadata.getInstance<Select>(`NRCRITERIOTAREFA_editable_${component.name.split('_')[2]}`);

        if (component.value) {
            selectCriterio.disabled = false;
            selectCriterio.datasource.setFilter({
                NRTAREFA: component.value,
            });
        } else {
            selectCriterio.disabled = true;
        }
    }

    public salvarTarefaChecklist({ component, event }: any) {
        event.preventDefault();
        const parametros = {
            NRATIVIDADE: component.name.split('_')[1],
            ROW: component.getEditedRows(),
        };

        if (Utilities.validateGrid(component)) {
            Utilities.save('/salvarRelacaoItemChecklist', parametros).then((response: any) => {
                if (response.data.length > 0) {
                    this.montarModalErroInatividade(response.data, false);
                }
                Utilities.showSuccessMessage();
                component.cancelEditedRows();
                component.datasource.get();
            });
        }
    }

    public dialogExcluirItemChecklist({ event, component }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00003002ItensController.excluirItemChecklist', 'Inativar Registros', 'Deseja realmente inativar todos os registros selecionados?');
        /* eslint-disable */
        this.codGridEditavel = component.name.split('_')[1];
        /* eslint-enable */
    }

    public excluirItemChecklist() {
        const grid = Metadata.getInstance<TekGrid>(`tek-grid-itens-com-atividades-criterios_${this.codGridEditavel}`);
        LoadingService.show();
        Http.post('/excluirItemChecklist', {
            NRATIVIDADE: this.codGridEditavel,
            ROW: grid.selectedRows,
        }).then((response: IDictionary) => {
            if (response.data.length > 0) {
                this.montarModalErroInatividade(response.data, true);
            }
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });

        grid.cancelEditedRows();
        grid.selectedRows = [];
        grid.datasource.get();
    }

    public setFilterGridTarefasChecklist({ component }: any) {
        component.datasource.setFilter({
            NRATIVIDADE: component.name.split('_')[1],
        });
    }

    public montarIdAtivo({ component }: any) {
        if (component.text === 'S') {
            component.text = '(Ativo)';
            component.cssStyle = 'font-size: 0.8375rem;';
        } else {
            component.text = '(Inativo)';
            component.cssStyle = 'font-size: 0.8375rem;';
        }
    }

    public montarBtnAtivo({ component }: any) {
        const ativo = Metadata.getInstance<Text>(`ativo-item_${component.name.split('_').pop()}`);
        if (ativo.text === '(Ativo)') {
            component.label = 'Inativar';
        } else {
            component.label = 'Ativar';
        }
    }
}
