import { Utilities } from '@/util/Utilities';
import {
    CheckboxMultiple, Form, Grid, Icon, LoadingService, Modal, ModalService, Select, SelectMultiple, Text,
    Textarea,
    TextInput,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, Metadata, SimpleDatasource,
} from '@zeedhi/core';
import { IterableCarousel, TekGrid } from '@zeedhi/teknisa-components-common';
import { IKanbanEventParam, Kanban, KanbanTask } from '@zeedhi/zd-kanban-common';
import { RichText } from '@zeedhi/zd-richtext-common';

export class FmKanbanTarefas {
    public modal!: Modal;

    public modalMoved!: Modal;

    public nrChecklistAgend: string = '';

    public currentModeloPop: any;

    public visibilidadeColunaResponsavel({ component }: any) {
        if (component.text.split(': ')[1] !== 'null') {
            component.isVisible = true;
        } else {
            component.isVisible = false;
        }
    }

    public visibilidadeColunaAtraso({ component }: any) {
        const situacao = component.name.split('_')[1];
        const atraso = component.name.split('_')[2];

        if (situacao === '1' && atraso !== '  ') {
            component.isVisible = true;
        } else {
            component.isVisible = false;
        }
    }

    public async criarModais() {
        this.modal = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00004_000_tarefas/modal/fm_00004_007_modal_info_tarefas' });
        this.modalMoved = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00004_000_tarefas/modal/fm_modal_mover_task' });
    }

    public destruirModais() {
        this.modal.destroy();
        this.modalMoved.destroy();
    }

    public async abrirModal({ component }: IEventParam<KanbanTask>) {
        /* eslint-disable */
        this.nrChecklistAgend = component.name.split('_')[0];
        const idsituacao = component.name.split('_')[1];
        /* eslint-enable */
        if (idsituacao !== '1') {
            this.modal.show();
            setTimeout(() => {
                this.montarInformacoesComplementares();
            }, 70);
        }
    }

    public fecharModal() {
        this.modal.hide();
    }

    public montarInformacoesComplementares() {
        LoadingService.show();
        const nr = this.nrChecklistAgend;
        const checkbox = Metadata.getInstance<CheckboxMultiple>('checkbox-itens-tarefas');
        const observacoes = Metadata.getInstance<Textarea>('observacoes-tarefas');
        const semItens = Metadata.getInstance<Text>('nao-ha-itens');
        const select = Metadata.getInstance<SelectMultiple>('select-epi-info');
        const carousel = Metadata.getInstance<IterableCarousel>('carousel-imagens-anexo-tarefas');
        let mensagem = '';
        this.loadModeloPopTarefa();

        select.datasource.setFilter({
            NRCHECKLISTAGEND: nr,
        });

        Http.post('/getItensTarefasKanban', {
            NRCHECKLISTAGEND: nr,
        }).then((response: IDictionary) => {
            const dados = response.data.data;

            select.setValue(select.datasource.data);

            checkbox.value = dados.filter((elemento: any) => elemento.IDREALIZADO === 'S').map((elemento: any) => elemento.NMTAREFA);
            (checkbox.datasource as SimpleDatasource)!.items = dados.map((elemento: any) => elemento.NMTAREFA);

            if (dados.length > 0) {
                semItens.isVisible = false;
            } else {
                semItens.isVisible = true;
            }

            if (dados[0].DSJUSTCANCELAMENTO !== null && dados[0].DSOBSTAREFAAGEND !== null) {
                mensagem = `Justificativa de cancelamento: ${dados[0].DSJUSTCANCELAMENTO} - Observações da Tarefa: ${dados[0].DSOBSTAREFAAGEND}`;
            } else if (dados[0].DSJUSTCANCELAMENTO !== null) {
                mensagem = `Justificativa de cancelamento: ${dados[0].DSJUSTCANCELAMENTO}`;
            } else if (dados[0].DSOBSTAREFAAGEND !== null) {
                mensagem = `Observações da Tarefa: ${dados[0].DSOBSTAREFAAGEND}`;
            } else {
                mensagem = 'Não há observações';
            }
            observacoes.value = mensagem;
            this.carregarFotos({ component: carousel });
            this.loadHistoricoTarefa();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public carregarFotos({ component }: any) {
        const nr = this.nrChecklistAgend;

        LoadingService.show();
        Http.post('/getFotosTarefas', {
            NRCHECKLISTAGEND: nr,
        }).then((response: any) => {
            if (response.data.data.length === 0) {
                component.isVisible = false;
                Metadata.getInstance<Text>('sem-resultado-imagens-tarefas').isVisible = true;
            } else {
                response.data.data.forEach((dados: IDictionary) => {
                    component.iterable.datasource.data.push({
                        NRFOTOTAREFAAGEND: dados.NRFOTOTAREFAAGEND,
                        DSFOTOTAREFAAGEND: dados.DSFOTOTAREFAAGEND,
                    });
                });
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public task: any;

    public openModalMoveTask({ task }: IKanbanEventParam) {
        if (task.columnFrom.id !== task.columnTo.id) {
            this.task = task;
            this.modalMoved.show();
            Metadata.getInstance<Form>('form-justificativa-moved-task').clearForm();
        }
    }

    public closeModalMoveTask(cancel: boolean = true) {
        if (cancel) {
            const kanban = Metadata.getInstance<Kanban>('kanban-tarefas');
            kanban.moveTask(this.task.task, { index: this.task.oldIndex, columnId: this.task.columnFrom.id });
        }
        this.modalMoved.hide();
    }

    public moveTask() {
        const form = Metadata.getInstance<Form>('form-justificativa-moved-task');
        const gridPrincipal = Metadata.getInstance<TekGrid>('tek-grid-tarefas');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/movedTask', {
                IDSITUACAOAGEND: this.task.columnTo.id,
                NRCHECKLISTAGEND: this.task.task.NRCHECKLISTAGEND,
                NMCHECKLISTAGEND: this.task.task.NMCHECKLISTAGEND,
                JUSTIFICATIVA: Metadata.getInstance<Textarea>('text-area-justificativa-moved-task').value,
            }).then((response: IDictionary) => {
                Utilities.showSuccessMessage();
                this.closeModalMoveTask(false);
                gridPrincipal.datasource.get();
                if (response.data) {
                    Utilities.alertMessage('Houve uma tentativa de reportar essa Tarefa. Porém, a mesma não possui Vistoria e por isso foi cancelada sem necessidade de aprovação prévia', 'warning');
                }
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public loadModeloPopTarefa() {
        const selectModeloPop = Metadata.getInstance<Select>('select-modelopop-info');
        const descricaoModeloPop = Metadata.getInstance<RichText>('descricao-modelo-pop');
        const linkModeloPop = Metadata.getInstance<TextInput>('url-modelo-pop');
        const btnBaixar = Metadata.getInstance<Icon>('download-modelo-pop');

        Http.get(`/getModelosPopTarefa?NRCHECKLISTAGEND=${this.nrChecklistAgend}`).then(async (response) => {
            const [modeloPop = {}] = response.data?.data ?? [];

            selectModeloPop.updateItems([modeloPop.NMMODELOPOP]);
            selectModeloPop.setValue(modeloPop.NMMODELOPOP);
            descricaoModeloPop.value = modeloPop.DESCMODELOPOP || '<p> Não há descrição </p>';
            linkModeloPop.value = modeloPop.LINKMODELOPOP || '';
            this.currentModeloPop = modeloPop;

            btnBaixar.disabled = !modeloPop.NMARQUIVO;
        });
    }

    public loadHistoricoTarefa() {
        const grid = Metadata.getInstance<Grid>('grid-historico-tarefas');
        grid.datasource.setFilter({
            NRCHECKLISTAGEND: this.nrChecklistAgend,
        });
    }

    public async downloadAnexoModeloPop() {
        try {
            if (!this.currentModeloPop.NMARQUIVO) {
                Utilities.showErrorMessage('Esse modelo de procedimento não tem arquivos');
                return;
            }

            const anexo = await fetch(`/api/getFile?fileName=${encodeURIComponent(this.currentModeloPop.NMARQUIVO)}`);

            if (!anexo.ok) {
                throw new Error(`Erro ao baixar o arquivo: ${anexo.statusText}`);
            }

            const fileBlob = await anexo.blob();
            const url = URL.createObjectURL(fileBlob);

            const a = document.createElement('a');
            a.href = url;
            a.download = this.currentModeloPop.NMARQUIVO || 'arquivo-desconhecido';
            document.body.appendChild(a);
            a.click();

            document.body.removeChild(a);
            URL.revokeObjectURL(url);
        } catch (error) {
            Utilities.showErrorMessage(`Erro ao baixar o arquivo: ${error}`);
        }
    }
}
