import { Utilities } from '@/util/Utilities';
import {
    DialogService, LoadingService, Modal, ModalService, TextInput,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, MemoryDatasource, Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Fm00040003CategoriaAuditoriaController {
    public modalErroExcluirCriterios!: Modal;

    public modalErroInativarCriterios!: Modal;

    constructor() {
        this.criarModais();
    }

    public async criarModais() {
        this.modalErroExcluirCriterios = await ModalService.createFromJson('fm_00040_000_auditoria/modal/modal_exclusao_categoria', true);
        this.modalErroInativarCriterios = await ModalService.createFromJson('fm_00040_000_auditoria/modal/modal_inatividade_categoria', true);
    }

    public async abrirModalErro() {
        if (this.modalErroExcluirCriterios) {
            await this.modalErroExcluirCriterios.show();
        } else {
            console.error("Modal 'modalErroExcluirCriterios' não foi inicializado.");
        }
    }

    public fecharModalErro() {
        if (this.modalErroExcluirCriterios) {
            this.modalErroExcluirCriterios.hide();
        }
    }

    public async abrirModalErroInativar() {
        if (this.modalErroInativarCriterios) {
            await this.modalErroInativarCriterios.show();
        } else {
            console.error("Modal 'modalErroInativarCriterios' não foi inicializado.");
        }
    }

    public fecharModalErroInativar() {
        if (this.modalErroInativarCriterios) {
            this.modalErroInativarCriterios.hide();
        }
    }

    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRCATEGORIAAUDIT: newId }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NMCATEGORIAAUDIT_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    /* eslint-disable */
    public canEditRow({ row }: any) {
        // return row['__added_row'] === true;
    }
    /* eslint-enable */

    public salvarCategoriaAuditoria({ component, event }: any) {
        event.preventDefault();
        const categoria = component.getEditedRows();

        if (categoria.length > 0) {
            if (categoria[0].NMCATEGORIAAUDIT != null) {
                LoadingService.show();
                Http.post('/salvarCategoriaAuditoria', categoria[0]).then((response: any) => {
                    if (response.data.status === 'erro') {
                        this.montarModalErroInativa(response.data.relacionados);
                    } else {
                        component.datasource.get();
                        Utilities.showSuccessMessage();
                    }
                }).catch((error: any) => {
                    Utilities.showErrorMessage(error);
                }).finally(() => {
                    component.cancelEditedRows();
                    LoadingService.hide();
                });
            } else {
                Utilities.alertMessage('Preencha o campo de "Nome"', 'error');
            }
        } else {
            Utilities.alertMessage('Preencha o campo de "Nome"!', 'error');
        }
    }

    public excluirCategoria({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00040003CategoriaAuditoriaController.aposConfirmarDelete', 'Excluir Registro(s)?');
    }

    public aposConfirmarDelete() {
        const component = Metadata.getInstance<TekGrid>('tek-grid-categoria-auditoria');
        LoadingService.show();
        Http.post('/excluirRegistros', {
            CATEGORIAS: component.selectedRows.map((element: any) => element.NRCATEGORIAAUDIT),
        }).then((response: IDictionary) => {
            if (response.data.status === 'erro') {
                this.montarModalErro(response.data.nao_excluidos);
                // Corrigido para usar "relacionados" do backend.
                // Utilities.showDialog(
                //     'Operação Bloqueada',
                //     'warning',
                //     response.data.mensagem || 'Algumas categorias não puderam ser excluídas.',
                // );
            } else {
                Utilities.showSuccessMessage();
            }

            component.datasource.get();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public async montarModalErro(relacionados: any) {
        await this.abrirModalErro();

        const grid = Metadata.getInstance<TekGrid>('grid-conflito-categoria');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        relacionados.forEach((rel: any) => {
            const dados = {
                NRMODELOAUDIT: rel.NRMODELOAUDIT,
                NMMODELOAUDIT: rel.NMMODELOAUDIT,
            };
            grid.datasource.post(dados);
        });

        await grid.datasource.get();
    }

    public async montarModalErroInativa(relacionados: any) {
        await this.abrirModalErroInativar();
        const grid = Metadata.getInstance<TekGrid>('grid-conflito-categoria-inativar');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        relacionados.forEach((rel: any) => {
            const dados = {
                NRMODELOAUDIT: rel.NRMODELOAUDIT,
                NMMODELOAUDIT: rel.NMMODELOAUDIT,
            };
            grid.datasource.post(dados);
        });

        await grid.datasource.get();
    }
}
