import {
    Button,
    FileInput, Form, LoadingService, Modal, ModalService, Select, SelectMultiple, Tabs, Text, TextInput, Textarea,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, Metadata,
} from '@zeedhi/core';
import {
    IterableCarousel, IterableComponentRender, TekGrid,
} from '@zeedhi/teknisa-components-common';
import { FieldGroup } from '@zeedhi/zd-field-group';
import { Utilities } from '@/util/Utilities';
import FileUpload from './FileUpload';

export class FmtOrdemServicoController {
    private visibilidade: boolean = true;

    private visibilidadeFrame: boolean = false;

    private modalCriarOrdemServico!: Modal;

    private carouselOrdemVisivel: boolean = false;

    private currentRow: any;

    public actionRow: any;

    public modalHistorico!: Modal;

    public mostrarDetalhesOrdem({ row }: any) {
        Metadata.getInstance<Text>('titulo-detalhes-ordens').text = row.NMORDEMOSERV;
        const inpEquipamento = Metadata.getInstance<TextInput>('input-equipamento-tab-detalhes-ordem');
        const inpCliente = Metadata.getInstance<TextInput>('input-cliente');
        const inpLocal = Metadata.getInstance<TextInput>('input-local');
        const inpCategoria = Metadata.getInstance<TextInput>('input-categoria-tab-detalhes-ordem');
        const inpTipo = Metadata.getInstance<TextInput>('input-tipo-manut-tab-detalhes-ordem');
        const inpPrioridade = Metadata.getInstance<TextInput>('input-prioridade-tab-detalhes-ordem');
        const inpStatus = Metadata.getInstance<TextInput>('input-status-tab-detalhes');
        const inpData = Metadata.getInstance<TextInput>('input-data-tab-detalhes');
        const inpEscopo = Metadata.getInstance<TextInput>('input-escopo-tab-detalhes-ordem');
        const txtDescricao = Metadata.getInstance<Textarea>('textarea-descricao-tab-detalhes');

        const equipe = Metadata.getInstance<Select>('input-equipe-tab-detalhes');
        const tecnico = Metadata.getInstance<SelectMultiple>('input-tecnico-tab-detalhes');
        const tecnicoExterno = Metadata.getInstance<SelectMultiple>('input-tecnico-externo-tab-detalhes');

        inpCliente.value = row.NMFANTCLIE;
        inpLocal.value = row.NMLOCALCLIENTE;
        inpEquipamento.value = row.NMEQUIPAMENTO;
        inpCategoria.value = row.NMCATEGORIAMANUT;
        inpTipo.value = row.NMTIPOMANUTENCAO;
        inpPrioridade.value = row.NMPRIORIDADEMANUT;
        inpStatus.value = row.NMIDSITUACAO;
        inpData.value = row.DTORDEMSERV;
        inpEscopo.value = row.NMESCOPOMANUT;
        txtDescricao.value = row.DSORDEMSERV;

        equipe.value = row.NREQUIPE;
        tecnico.setValue(row.CDOPERADOR, false);
        tecnicoExterno.setValue(row.NRPROFISSIONALEXTERNO.map(Number), false);

        this.visibilidade = false;
        this.visibilidadeFrame = !this.visibilidade;
        this.currentRow = row;

        equipe.isVisible = (this.hasValue(row.NREQUIPE));
        tecnico.isVisible = (this.hasValue(row.CDOPERADOR));
        tecnicoExterno.isVisible = (this.hasValue(row.NRPROFISSIONALEXTERNO));

        this.montaDadosComentarios();
    }

    /* eslint-disable */
    public hasValue(input: number | string | any[]): boolean {
        if (Array.isArray(input)) {
          // Verifica se o array não está vazio e não contém apenas valores vazios ou strings vazias
          return input.some((item) => item !== null && item !== undefined && item.toString().trim().length > 0);
        } else if (typeof input === 'number') {
          // Verifica se o número não é NaN
          return !isNaN(input);
        } else if (typeof input === 'string') {
          // Verifica se a string não está vazia ou só tem espaços
          return input.trim().length > 0;
        }
        return false; // Caso não seja array, número ou string
      }
      
    /* eslint-enable */

    public montaDadosComentarios() {
        const component = Metadata.getInstance<Tabs>('tabs-detalhes-ordem');
        if (component.tabs[component.activeTab].name === 'tab-comentarios-ordem') {
            setTimeout(() => {
                const cardsComentarios = Metadata.getInstance<IterableComponentRender>('iterable-cards-comentarios');
                cardsComentarios.datasource.setFilter({
                    NRORDEMSERV: this.currentRow.NRORDEMSERV,
                });
            }, 500);
        }
    }

    public saveComentarioOrdem() {
        const form = Metadata.getInstance<Form>('form-comentario-tab');

        if (Utilities.validateForm(form)) {
            Http.post('/saveComentarioOrdem', {
                NRORDEMSERV: this.currentRow.NRORDEMSERV,
                DSCOMENTARIO: form.value.textareaComentarios,
            }).then(() => {
                Utilities.showSuccessMessage();
                Metadata.getInstance<Textarea>('textareaComentarios').value = null;
                Metadata.getInstance<IterableComponentRender>('iterable-cards-comentarios').datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            });
        }
    }

    public voltarTelaOrdemServico() {
        this.visibilidade = true;
        this.visibilidadeFrame = !this.visibilidade;
    }

    public async criarOrdemServico() {
        this.modalCriarOrdemServico?.destroy();
        this.modalCriarOrdemServico = await ModalService.createFromJsonPost('', { jsonPath: 'fmt_ordem_servico/modal/fmt_criar_ordem_servico' });
        this.modalCriarOrdemServico.show();
    }

    public fecharModalCriarOrdem() {
        this.modalCriarOrdemServico?.destroy();
        this.modalCriarOrdemServico.hide();
    }

    public mudaVisibilidadeTecnicos({ component }: any) {
        const selectTecnicos = Metadata.getInstance<Select>('selectTecnicos');
        selectTecnicos.isVisible = component.value === 'N';
    }

    public async abrirModalHistorico() {
        this.modalHistorico?.destroy();
        this.modalHistorico = await ModalService.createFromJson('fmt_ordem_servico/modal/fmt_historico_ordemserv', true);
        this.modalHistorico.show();
    }

    public fecharModalHistorico() {
        this.modalHistorico.hide();
    }

    public async saveOrdemServico() {
        const form = Metadata.getInstance<Form>('form-criar-ordem-servico');
        const imgs = await this.updloadPictures('file-input-fotos-ordem');
        const grid = Metadata.getInstance<TekGrid>('grid-ordem-servico');

        const tipoResponsavel = Metadata.getInstance<Select>('tipo-responsavel');
        const profissionalE = Metadata.getInstance<Select>('profissional-Externo');
        let cadastroValido: boolean = false;
        let novoProfissional = {};

        if (tipoResponsavel.value === '3') {
            if (profissionalE.datasource.data.length === 0) {
                profissionalE.removeValidation('required');
                cadastroValido = Metadata.getInstance<Form>('form-tecnico-externo').validate();

                if (cadastroValido) {
                    novoProfissional = {
                        NMPROFISSIONALEXTERNO: Metadata.getInstance<TextInput>('nome-tecnico-externo').value,
                        NMEMPRESA: Metadata.getInstance<TextInput>('nome-empresa-externo').value,
                        INSCRICAO: Metadata.getInstance<TextInput>('cpf-tecnico-externo').value,
                    };
                } else {
                    return;
                }
            }
        } else {
            cadastroValido = true;
        }

        if (Utilities.validateForm(form) && cadastroValido) {
            LoadingService.show();
            Http.post('/saveOrdemServico', {
                NMORDEMOSERV: form.value.inputNome,
                IDSITUACAO: form.value.selectStatus,
                NREQUIPAMENTO: form.value.selectEquipamento,
                NRCATEGORIAMANUT: form.value.selectCategoria,
                NRTIPOMANUTENCAO: form.value.selectTipoManutencao,
                NRPRIORIDADEMANUT: form.value.selectPrioridade,
                DTORDEMSERV: form.value.dateData,
                IDFUNCEXTERNO: form.value.selectProfiExterno,
                TECNICOS: form.value.selectTecnicos,
                DSORDEMSERV: form.value.textDescricao,
                NRESCOPOS: form.value.selectEscopo,
                FOTOS: imgs,
                DSCOMENTARIO: form.value.textComentario,
                NRPROFISSIONALEXTERNO: profissionalE.value,
                NREQUIPE: Metadata.getInstance<Select>('EQUIPE').value,
                CDOPERADOR: Metadata.getInstance<SelectMultiple>('OPERADOR').value,
                NRLOCALCLIENTE: Metadata.getInstance<Select>('NRLOCALCLIENTE-criar-ordem').value,
                NOVOPROFISSIONAL: novoProfissional,
                NREPI: Metadata.getInstance<Select>('select-epi').value,
                NRMODELOPOP: Metadata.getInstance<Select>('select-modelopop').value,
            }).then(() => {
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                this.fecharModalCriarOrdem();
                LoadingService.hide();
            });
        }
    }

    public mostrarFotoOrdem({ component }: any) {
        const carouselFotos = Metadata.getInstance<IterableCarousel>('carouselFotosOrdem');
        carouselFotos.iterable.datasource.updateData([]);
        const imagens: any = [];
        component.value.forEach((element: any, pos: any) => {
            imagens.push({
                id: `numero_${pos}`,
                src: window.URL.createObjectURL(element),
            });
        });
        carouselFotos.iterable.datasource.updateData(imagens);
        this.carouselOrdemVisivel = component.value.length > 0;
    }

    public async updloadPictures(name: string) {
        const fileInputObject = Metadata.getInstance<FileInput>(name).value;
        let arquivos: any = [];
        if (fileInputObject && fileInputObject.length > 0) {
            const types: IDictionary = ['image/jpg', 'image/jpeg', 'image/png'];
            const buildFilesPromises = fileInputObject.map((file: any) => {
                if (types.includes(file.type)) {
                    return FileUpload.buildFileSendObject(file);
                }
                return [];
            });
            await Promise.all(buildFilesPromises).then((files: any) => {
                arquivos = files;
            });
        }
        return arquivos;
    }

    public defineRowAction({ row }: any) {
        this.actionRow = row;
    }

    public async abrirHistorico() {
        await this.abrirModalHistorico();
        Metadata.getInstance<TekGrid>('grid-historico-ordemserv').datasource.setFilter({
            NRORDEMSERV: this.actionRow.NRORDEMSERV,
        });
    }

    public changeResponsavel({ component }: IEventParam<Select>) {
        const elements = {
            operador: Metadata.getInstance<SelectMultiple>('OPERADOR'),
            equipe: Metadata.getInstance<Select>('EQUIPE'),
            profissionalE: Metadata.getInstance<Select>('profissional-Externo'),
            fielGroupTecnico: Metadata.getInstance<FieldGroup>('field-group-cadastro-tecnico-externo'),
            local: Metadata.getInstance<Select>('NRLOCALCLIENTE-criar-ordem'),

            formTecnicoExterno: Metadata.getInstance<Form>('form-tecnico-externo'),
            btnNovo: Metadata.getInstance<Button>('btn-novo-profissional-externo'),
            nome: Metadata.getInstance<TextInput>('nome-tecnico-externo'),
            empresa: Metadata.getInstance<TextInput>('nome-empresa-externo'),
            cnpj: Metadata.getInstance<TextInput>('cpf-tecnico-externo'),
        };

        // Resetar todos os elementos para estado inicial
        elements.btnNovo.label = 'Novo';
        const resetAll = () => {
            this.setElementState(elements.operador, false);
            this.setElementState(elements.equipe, false);
            this.setElementState(elements.profissionalE, false);
            elements.btnNovo.isVisible = false;
            elements.fielGroupTecnico.isVisible = false;
            elements.formTecnicoExterno.clearForm();
            this.setElementState(elements.nome, true, null, null);
            this.setElementState(elements.empresa, true, null, null);
            this.setElementState(elements.cnpj, true, null, null);
        };

        // Aplicar lógica com base no valor do componente
        resetAll();
        switch (component.value) {
            case '1':
                this.setElementState(elements.operador, true, undefined, { type: 'required', params: {} });
                elements.operador.datasource.setFilter({ NRLOCALCLIENTE: elements.local.value });
                break;
            case '2':
                this.setElementState(elements.equipe, true, undefined, { type: 'required', params: {} });
                elements.equipe.datasource.setFilter({ NRLOCALCLIENTE: elements.local.value });
                break;
            case '3':
                this.setElementState(elements.profissionalE, true, undefined, { type: 'required', params: {} });
                this.setElementState(elements.btnNovo, true);
                break;
            default:
                // Default logic handled by resetAll
                break;
        }
    }

    public visibilidadeFieldGroupProfissionalExterno({ component }: IEventParam<Button>) {
        const elements = {
            formTecnicoExterno: Metadata.getInstance<Form>('form-tecnico-externo'),
            fielGroupTecnico: Metadata.getInstance<FieldGroup>('field-group-cadastro-tecnico-externo'),
            nome: Metadata.getInstance<TextInput>('nome-tecnico-externo'),
            empresa: Metadata.getInstance<TextInput>('nome-empresa-externo'),
            cnpj: Metadata.getInstance<TextInput>('cpf-tecnico-externo'),
        };

        const resetAll = () => {
            this.setElementState(elements.nome, true);
            this.setElementState(elements.empresa, true);
            this.setElementState(elements.cnpj, true);
            component.label = 'Novo';
            elements.fielGroupTecnico.isVisible = false;
            elements.formTecnicoExterno.clearForm();
        };

        if (component.label === 'Novo') {
            elements.fielGroupTecnico.isVisible = true;
            this.setElementState(elements.nome, true, undefined, { type: 'required', params: {} });
            this.setElementState(elements.empresa, true, undefined, { type: 'required', params: {} });
            this.setElementState(elements.cnpj, true, undefined, { type: 'required', params: {} });
            component.label = 'Cancelar';
        } else {
            resetAll();
        }
    }

    // Função auxiliar para alterar visibilidade e valor
    public setElementState(element: any, isVisible: boolean, value: any = null, validation: { type: string; params: any } | null = null) {
        element.isVisible = isVisible;
        if (value !== undefined) {
            element.value = value;
        }
        if (validation) {
            element.addValidation(validation.type, validation.params);
        } else {
            element.removeValidation('required');
        }
    }

    public changeTipoDeLocal({ component }: IEventParam<Select>) {
        const tipoLocal = Metadata.getInstance<Select>('NRTIPOLOCALCLI-criar-ordem');
        const local = Metadata.getInstance<Select>('NRLOCALCLIENTE-criar-ordem');

        tipoLocal.value = null;
        local.value = null;

        if (component.value) {
            tipoLocal.disabled = false;
            tipoLocal.datasource.setFilter({
                CDCLIENTE: component.value,
            });
        } else {
            tipoLocal.disabled = true;
            local.disabled = true;
        }
    }

    public changeLocal({ component }: IEventParam<Select>) {
        const local = Metadata.getInstance<Select>('NRLOCALCLIENTE-criar-ordem');
        const responsavel = Metadata.getInstance<Select>('tipo-responsavel');

        if (component.value) {
            local.disabled = false;
            responsavel.disabled = false;

            local.datasource.setFilter({
                NRTIPOLOCALCLI: component.value,
                UNICA: 'S',
                CDCLIENTE: Metadata.getInstance<Select>('CDCLIENTE-criar-ordem').value,
            });

            this.changeResponsavel({ component: responsavel });
        } else {
            local.value = null;
            local.disabled = true;
            responsavel.value = null;
            responsavel.disabled = true;
        }
    }
}
